/*******************************************************************************
  app-timers.h

  Timer definition prototypes and definitions useful for an application.

  jim@ember.com
  Copyright 2004 by Ember Corporation. All rights reserved.                 *80*
*******************************************************************************/

//******************************************************************************
// Public Timer definitions

// Total number of timers available to the application.
// This can be defined in the application before including this file, in case
// the application needs to use a different number of timers.
#if !defined(NUM_TIMERS)
  #define NUM_TIMERS 4
#endif
#if !NUM_TIMERS
  #error NUM_TIMERS = 0!
#endif

//******************************************************************************
// Private Timer definitions

// We want 16 system ticks per second (period = 62.5 mS)
// AVR clock: 1 second = 1024 clocks
// Net clock: 1 second = 31250 clocks

#define SYSTEM_CLOCK_PERIOD     64

// Application timer function prototype.
typedef void (*TimerFunctionPointer)(int8u timerId, int32u timerParam);

// Timer structure definition
typedef struct timer_t {
  int8u timerId;
  int16u expiration;
  TimerFunctionPointer timer_function;
  int32u param;
  boolean recurring;
  int16u recurValue;
} timer;

//******************************************************************************
// Public Timer function prototypes

void initTimers(void);
void addTimer(int8u timerId,      // Timer # (0..NUM_TIMERS-1)
              TimerFunctionPointer function, // Function to call on timeout
              int32u param,       // Parameter to send to function
              int16u ticks,       // Number of sysTicks to timeout/recur
              boolean recurring); // Specify if timer will keep recurring
void resetTimerClock(void);
void handleTimers(void);
int32u msToNextEvent(void);


//******************************************************************************
