/*

This file contains commands sent from nick, such as add a serial number.   This file will look at the serial
number nick gives me and determine the device class SENSOR/MOBILE/HANDHELD based on the byte in the serial number

*/

//Serial Number is built up as:
//
//		0			1		2	3		4  5    	6   7   8   9
//	Country of	  Device    Month		Year		No. In Batch
// 	  origin	  Class
//
//E.x. CA05080001   This would be a serial number for a PST unit made in China
//
//C = China, U = UK
//Device Class = See defines Below

#define DEV_LOC				1			//Device Class (see above)
#define DEVICE_PST			'A'			//these will all be SENSORS
#define DEVICE_RH			'C'
#define DEVICE_PT100		'E'
#define DEVICE_AUDITORRF	'G'
#define DEVICE_mA			'M'			//RF515 mA transmitter added 15-10-08 was I but now changed to M 08-04-09
#define DEVICE_TRANSPORT	'T'


void LoadSN(int8u *serialnumber, int8u resendlastradio);
void CheckNloadSN(int8u *serialnumber, int8u Dev_Type, int8u resendlastradio);
int8u CheckSerialNumber(int8u *serialnumber, int8u Dev_Type);
void AddNewSN(int8u index, int8u *serialnumber, int8u Dev_Type, int8u resendlastradio);
void RemoveSN(int8u *serialnumber);
void RemoveDevice(void);

void LoadSN(int8u *serialnumber, int8u resendlastradio)  //will validate serial number before trying to enter it.
{
	int i;

	for(i=2;i<10;i++){
		if(!isdigit(serialnumber[i])){
			return;					//this is not a valid serial num will not add
		}
	}

	switch(serialnumber[DEV_LOC])
	{
		case DEVICE_PST:
		case DEVICE_RH:
		case DEVICE_PT100:
		case DEVICE_mA:
			//find free location in serial number array and load this new sn in
			//fprintf(stdout, "Got here\n\r");
			CheckNloadSN(serialnumber, SENSOR, resendlastradio);
			break;

		case DEVICE_TRANSPORT:
			//find free location in serial number array and load this new sn in
			CheckNloadSN(serialnumber, MOBILE, resendlastradio);
			break;

		case DEVICE_AUDITORRF:
			//find free location in serial number array and load this new sn in
			CheckNloadSN(serialnumber, HANDHELD, resendlastradio);
			break;
	}
}

void SetforRemoval(int8u *serialnumber)
{
	int8u i = 0;

	switch(serialnumber[DEV_LOC])
	{
		case DEVICE_PST:
		case DEVICE_RH:
		case DEVICE_PT100:
		case DEVICE_mA:
			//set for removal
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(String_Compare(serialnumber, 0, SensorSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					//found device
					if((!SensorSNid[i].Remove) && (!SensorSNid[i].Removed)) SensorSNid[i].Remove = TRUE;
					if(String_Compare(SensorSNid[i].Eui64number, 0, unreg, 0, 6)){	//if unregistered remove
						RemoveSno(SensorSNid[i].SerialNumber);
						RemoveSN(SensorSNid[i].SerialNumber);		//delete from info store
					}
					break;
				}
			}
			break;

		case DEVICE_TRANSPORT:
			//set for removal
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(String_Compare(serialnumber, 0, MobileSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					//found device
					if((!MobileSNid[i].Remove) && (!MobileSNid[i].Removed)) MobileSNid[i].Remove = TRUE;
					if(String_Compare(MobileSNid[i].Eui64number, 0, unreg, 0, 6)){	//if unregistered remove
						RemoveSno(MobileSNid[i].SerialNumber);
						RemoveSN(MobileSNid[i].SerialNumber);		//delete from info store
					}
					break;
				}
			}
			break;

		case DEVICE_AUDITORRF:
			//set for removal
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(String_Compare(serialnumber, 0, HandheldSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					//found device
					if((!HandheldSNid[i].Remove) && (!HandheldSNid[i].Removed)) HandheldSNid[i].Remove = TRUE;
					if(String_Compare(HandheldSNid[i].Eui64number, 0, unreg, 0, 6)){	//if unregistered remove
						RemoveSno(HandheldSNid[i].SerialNumber);
						RemoveSN(HandheldSNid[i].SerialNumber);		//delete from info store
					}
					break;
				}
			}
			break;
	}

}

void RemoveDevice(void)
{
	int8u DeviceThatsBeenRemoved;
	int8u binding_address;

	if(SensorRemoved){	//this will only remove one s/n at a time
		DeviceThatsBeenRemoved = SearchDeviceArrayForRemoved(SENSOR);
		if(DeviceThatsBeenRemoved == 254) {
			Error_Note("RemovallS Error", 0, "N/A");	//debug
		}
		else{
			RemoveSno(SensorSNid[DeviceThatsBeenRemoved].SerialNumber);//tell Nick i have removed that serial number
			binding_address = find_sensor_binding(SensorSNid[DeviceThatsBeenRemoved].Eui64number); //find on binding table
			if(binding_address != 254) emberDeleteBinding(binding_address); 	//delete from binding table
			RemoveSN(SensorSNid[DeviceThatsBeenRemoved].SerialNumber);		//delete from info store
		}
		SensorRemoved = FALSE;
	}

	if(MobileRemoved){
		DeviceThatsBeenRemoved = SearchDeviceArrayForRemoved(MOBILE);
		if(DeviceThatsBeenRemoved == 254) {
			fprintf(stdout, "RemovallM Error\n\r");
		}
		else{
			RemoveSno(MobileSNid[DeviceThatsBeenRemoved].SerialNumber);//tell Nick i have removed that serial number
			binding_address = find_sensor_binding(MobileSNid[DeviceThatsBeenRemoved].Eui64number);
			if(binding_address != 254) emberDeleteBinding(binding_address); 	//delete from binding table
			RemoveSN(MobileSNid[DeviceThatsBeenRemoved].SerialNumber);
		}
		MobileRemoved = FALSE;
	}

	if(HandheldRemoved){
		DeviceThatsBeenRemoved = SearchDeviceArrayForRemoved(HANDHELD);
		if(DeviceThatsBeenRemoved == 254) {
			fprintf(stdout, "RemovallH Error\n\r");
		}
		else{
			RemoveSno(HandheldSNid[DeviceThatsBeenRemoved].SerialNumber);//tell Nick i have removed that serial number
			binding_address = find_sensor_binding(HandheldSNid[DeviceThatsBeenRemoved].Eui64number);
			if(binding_address != 254) emberDeleteBinding(binding_address); 	//delete from binding table
			RemoveSN(HandheldSNid[DeviceThatsBeenRemoved].SerialNumber);
		}
		HandheldRemoved = FALSE;
	}


}



void RemoveSN(int8u *serialnumber) //will remove a sensor and delete all records of it
{
	int8u Location = 0;

	switch(serialnumber[DEV_LOC])
	{
		case DEVICE_PST:
		case DEVICE_RH:
		case DEVICE_PT100:
		case DEVICE_mA:
			//find serial number in array
			for(Location=0;Location<MAX_SENSORS;Location++) {
				if(String_Compare(serialnumber, 0, SensorSNid[Location].SerialNumber, 0, SIZE_SNUM))  break; //
			}
			memset(&SensorSNid[Location], 0, sizeof(DeviceInformation));
			break;

		case DEVICE_TRANSPORT:
			//find serial number in array
			for(Location=0;Location<MAX_MOBILE;Location++) {
				if(String_Compare(serialnumber, 0, MobileSNid[Location].SerialNumber, 0, SIZE_SNUM))  break; //
			}
			memset(&MobileSNid[Location], 0, sizeof(DeviceInformation));
			break;

		case DEVICE_AUDITORRF:
			//find serial number in array
			for(Location=0;Location<MAX_HANDHELD;Location++) {
				if(String_Compare(serialnumber, 0, HandheldSNid[Location].SerialNumber, 0, SIZE_SNUM))  break; //
			}
			memset(&HandheldSNid[Location], 0, sizeof(DeviceInformation));
			break;
	}
}


//if no room avaliable will not error but will also not add the number
void CheckNloadSN(int8u *serialnumber, int8u Dev_Type, int8u resendlastradio)
{
	int8u i = 0;

	switch(Dev_Type)
	{
		case SENSOR:
			if(CheckSerialNumber(serialnumber, SENSOR)){	//returns true if not had this serial number before
				//should drop out if i becomes greater than MAX_SENSORS
				while(SensorSNid[i].SerialNumber[0] && (i<MAX_SENSORS)) i++;
				//fprintf(stdout, "I is %d\n\r", i);			//Debug
				if(i < MAX_SENSORS) AddNewSN(i, serialnumber, Dev_Type, resendlastradio);
			}
			break;

		case MOBILE:
			if(CheckSerialNumber(serialnumber, MOBILE)){
				while(MobileSNid[i].SerialNumber[0] && (i<MAX_MOBILE)) i++;

				if(i < MAX_MOBILE) AddNewSN(i, serialnumber, Dev_Type, resendlastradio);
			}
			break;

		case HANDHELD:
			if(CheckSerialNumber(serialnumber, HANDHELD)){
				while(HandheldSNid[i].SerialNumber[0] && (i<MAX_HANDHELD)) i++;

				if(i < MAX_HANDHELD) AddNewSN(i, serialnumber, Dev_Type, resendlastradio);
			}
			break;
	}
}

int8u CheckSerialNumber(int8u *serialnumber, int8u Dev_Type)//checks to see if we have this serial number already
{															//returns TRUE if does not find it
	int8u i = 0;

	switch(Dev_Type)
	{
		case SENSOR:
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(String_Compare((serialnumber), 0, SensorSNid[i].SerialNumber, 0, SIZE_SNUM)) {
				//Found Serial number already don't load
					//fprintf(stdout, "fOUND SNO WILL NOT LOAD\n\r");
					return(FALSE); //
				}
				//else return(TRUE); //have not Found Serial number already load
			}
			return(TRUE); //have not Found Serial number already load
			//break;

		case MOBILE:
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(String_Compare((serialnumber), 0, MobileSNid[i].SerialNumber, 0, SIZE_SNUM)) {
				//Found Serial number already don't load
					return(FALSE); //
				}
				//else return(TRUE); //have not Found Serial number already load
			}
			return(TRUE); //have not Found Serial number already load
			//break;

		case HANDHELD:
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(String_Compare((serialnumber), 0, HandheldSNid[i].SerialNumber, 0, SIZE_SNUM)) {
				//Found Serial number already don't load
					return(FALSE); //
				}
				//else return(TRUE); //have not Found Serial number already load
			}
			return(TRUE); //have not Found Serial number already load
			//break;
	}
	return(FALSE); //
}


void AddNewSN(int8u index, int8u *serialnumber, int8u Dev_Type, int8u resendlastradio)
{
	switch(Dev_Type)
	{
		case SENSOR:
			String_Copy(serialnumber, 0, SensorSNid[index].SerialNumber, 0, SIZE_SNUM);
			String_Copy(unreg, 0, SensorSNid[index].Eui64number, 0, 6);
			SensorSNid[index].AbnormalGatewayShutdown = resendlastradio;
			break;

		case MOBILE:
			String_Copy(serialnumber, 0, MobileSNid[index].SerialNumber, 0, SIZE_SNUM);
			String_Copy(unreg, 0, MobileSNid[index].Eui64number, 0, 6);
			break;

		case HANDHELD:
			String_Copy(serialnumber, 0, HandheldSNid[index].SerialNumber, 0, SIZE_SNUM);
			String_Copy(unreg, 0, HandheldSNid[index].Eui64number, 0, 6);
			break;
	}
}









