/*
This file contains the code to handle incomming data packets from a device
*/


//-----------------------------------------------------
void PrintData_file(int8u Sensor); //debug stuff only
//-----------------------------------------------------

int8u CheckCmdMsg_Result;
boolean TaskFirmwareTimer = FALSE;
boolean Duplicate = FALSE;
boolean BadPacketNumber = FALSE;



void IncommingSensorData(int8u *contents);
boolean SendCmds(int8u *SerialNum, int8u *OutgoingCmd, int8u *CmdStore);
void DataPacket1(int8u *contents, int8u Device, int8u type);
void DataPacket2(int8u *contents, int8u Device, int8u type);
void DataPacket3(int8u *contents, int8u Device, int8u type);
void DataPacket4(int8u *contents, int8u Device, int8u type);
void SendToNick(int8u Device);
void GetSequence(int8u Device, int8u PacketNumber, int8u SequenceNumber);


void IncommingSensorData(int8u *contents)
{
	EmberBindingTableEntry binding;
	int8u i, Device, ChkCmdResult, Error;
	int8u CmdBuffer[100];
	int8u ReplyBuffer[90];
	boolean SendCmd = FALSE;
	boolean GotFullMb = FALSE;
	EmberStatus status;
	EmberMessageBuffer reply = 0;
	//EmberCost LinkCost;


	//first check to see if this is one of our nodes
    for (i = 0; i < MAX_METERS; i++)
	{
      emberGetBinding(i, &binding);
      if (MEMCOMPARE(binding.identifier, (contents+1), 8) == 0)
	  {
		//we should note the senders binding to update the cost and route
		//to the Device.
		//printf("Updating Cost\n\r");    //debug info
	    emberNoteSendersBinding(i);
		break;
      }
    }

	//get eui64 number from packet
	String_Copy(contents, EUI64_START, Cipher_Params_CU.Eui64, 0, SIZE_EUI64);
	//find this sensor in our array
	Device = SearchDeviceArray(SENSOR);
	String_Copy(SensorSNid[Device].SerialNumber, 0, Cipher_Params_CU.SerialNum, 0, SIZE_SNUM);
	//check the shutdown flag, if set now clear as the tx is now sending back data.
	if(SensorSNid[Device].AbnormalGatewayShutdown){
		SensorSNid[Device].AbnormalGatewayShutdown = FALSE;
		ResetResendFlag(SensorSNid[Device].SerialNumber);	//tell Nick
	}

	if(the_time >= SensorSNid[Device].LastPacketRecieved)//if its been longer than 7 sec's since last herd form
	{													  //this transmitter
		SensorSNid[Device].Packet1 = 0;
		SensorSNid[Device].Packet2 = 0;					  //reset all the flags as this must be a new Monster Block
		SensorSNid[Device].Packet3 = 0;
		SensorSNid[Device].Packet4 = 0;
		SensorSNid[Device].RequiredPackets = 0;
		MEMSET(SensorSNid[Device].Data,0,MAX_DATASIZE);  //clear down the data
		//printf("New MB Rx'd\n\r");
		//reset the sequence number flags here!!!
		SensorSNid[Device].Pkt1SeqNum = 255;
		SensorSNid[Device].Pkt2SeqNum = 255;
		SensorSNid[Device].Pkt3SeqNum = 255;
		SensorSNid[Device].Pkt4SeqNum = 255;
		SensorSNid[Device].GotSequence = FALSE;
	}

	//SensorSNid[Device].LastPacketRecieved = the_time + DataPacketTimeOut;	  //reset the timout

	switch(contents[0])
	{
		case SENSOR_DATAPACKET_START:
			DataPacket1(contents, Device, SENSOR);
			break;

		case SENSOR_DATAPACKET_2:
			DataPacket2(contents, Device, SENSOR);
			break;

		case SENSOR_DATAPACKET_3:
			DataPacket3(contents, Device, SENSOR);
			break;

		case SENSOR_DATAPACKET_4:
			DataPacket4(contents, Device, SENSOR);
			break;
	}

	//printf("####Recieving Data %i from %s####\n\r",SensorSNid[Device].Data[BLKLEN_LOC],SensorSNid[Device].SerialNumber);	//debug only
	//printf("Flags pkt1 %i, Flags pkt2 %i, Flags pkt3 %i, Flags pkt4 %i, Required Pkts %i",SensorSNid[Device].Packet1,SensorSNid[Device].Packet2,SensorSNid[Device].Packet3,SensorSNid[Device].Packet4,SensorSNid[Device].RequiredPackets);


	if(!Duplicate && !BadPacketNumber)	//should ensure a monster block only gets recieved and passed to nick the once
	{
		if((SensorSNid[Device].RequiredPackets == 1) &&  SensorSNid[Device].Packet1) GotFullMb = TRUE;
		if((SensorSNid[Device].RequiredPackets == 2) && (SensorSNid[Device].Packet1) && (SensorSNid[Device].Packet2)) GotFullMb = TRUE;
		if((SensorSNid[Device].RequiredPackets == 3) && (SensorSNid[Device].Packet1) && (SensorSNid[Device].Packet2) && (SensorSNid[Device].Packet3)) GotFullMb = TRUE;
		if((SensorSNid[Device].RequiredPackets == 4) && (SensorSNid[Device].Packet1) && (SensorSNid[Device].Packet2) && (SensorSNid[Device].Packet3) && (SensorSNid[Device].Packet4)) GotFullMb = TRUE;

		//if((SensorSNid[Device].Packet1+SensorSNid[Device].Packet2+SensorSNid[Device].Packet3+SensorSNid[Device].Packet4) == SensorSNid[Device].RequiredPackets)
		if(GotFullMb)
		{
			//dump that data to nick
			//fprintf(stdout, "\n\r####Recieved Monster Block of length %i from %s####\n\r",SensorSNid[Device].Data[BLKLEN_LOC],SensorSNid[Device].SerialNumber);	//debug only
			//printf("\n\r####Recieved Monster Block of length %i from %s####\n\r",SensorSNid[Device].Data[BLKLEN_LOC],SensorSNid[Device].SerialNumber);	//debug only
			SensorSNid[Device].GotMonsterBlock = TRUE;
			SensorSNid[Device].Packet1 = 0;
			SensorSNid[Device].Packet2 = 0;
			SensorSNid[Device].Packet3 = 0;
			SensorSNid[Device].Packet4 = 0;
			SensorSNid[Device].RequiredPackets = 0;
			SensorSNid[Device].GotSequence = FALSE;
			//do not reset the sequence number flags here!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

			//this is where we will ack the data packet and look for cmds from nick to the Sensor
			//Send data to Nick
			if(!SensorSNid[Device].Remove)	//check to see if this is set for removal don't waste time saving data if set for removal
			{
				//status = emberGetCostToSender(&LinkCost);    //*********New link strength information link.    Strength outbound to the transmitter.
				//printf("Ember Cost is %x\n\r",LinkCost);    //higher the cost the weaker the signal path this could be passed to Nick in DoRxCmd  //debug info
				Error = DoRxCmd(SensorSNid[Device].SerialNumber, SensorSNid[Device].Data);
				if(Error != NOERROR)
				{
					//ReplyBuffer[0] = RESENDLAST_MONSTERBLOCK;
					//reply = emberFillLinkedBuffers(ReplyBuffer, 1); //1 is My overheads MSG_TYPE
					//status = emberSendReply(reply);
					//emberReleaseMessageBuffer(reply);
					CheckEmberStatus(status);
					SensorSNid[Device].GotMonsterBlock = FALSE;  //get ready for a new one
					//fprintf(stdout, "Data Save Error\n\r");
					Error_Note("ERROR RETURNED FROM DoRxCmd() **CODE", Error, SensorSNid[Device].SerialNumber);
					Error_Stop("ERROR RETURNED FROM DoRxCmd() **CODE", Error, SensorSNid[Device].SerialNumber);
					return;
				}
				else
				{
					SensorSNid[Device].GotMonsterBlock = FALSE;//MEMSET(SensorSNid[Device].Data,0,MAX_DATASIZE);  //clear down the data
				}
			}

			//status = emberGetLastHopLqi(SensorSNid[Device].Lqi);
			//fprintf(stdout, "LQI is %i\n\r", SensorSNid[Device].Lqi);	//would pass to nick here
			//Check if the Transmitter is due to be removed
			if(SensorSNid[Device].Remove){
					//fprintf(stdout, "Sending Remove\n\r");	//debug
					ReplyBuffer[0] = CU_REMOVE;
					MEMCOPY(&ReplyBuffer[1], SensorSNid[Device].Eui64number, EUI_LEN);	//add dest eui address need to find it later em msg sent
					ReplyBuffer[9] = SENSOR;
					reply = emberFillLinkedBuffers(ReplyBuffer, 10);
					status = emberSendDatagram(SensorSNid[Device].BindingTableIndex, reply);
					emberReleaseMessageBuffer(reply);
					CheckEmberStatus(status);
					return;
			}
			else
			{
				SendCmd = SendCmds(SensorSNid[Device].SerialNumber, ReplyBuffer, CmdBuffer);
			}
		}
	}
	else
	{
		Duplicate = FALSE;
		BadPacketNumber = FALSE;
	}


	//send the cmd back to the transmitter
	if(SendCmd){
			reply = emberFillLinkedBuffers(ReplyBuffer, (CmdBuffer[0] + 1) +1); //1 is My overheads MSG_TYPE
			status = emberSendReply(reply);
			emberReleaseMessageBuffer(reply);
			CheckEmberStatus(status);
	}
	//else{
        //Send Dummy Packet for LQI Purposes
        //ReplyBuffer[0] = LQIDUMMYPACKET;
		//reply = emberFillLinkedBuffers(ReplyBuffer, 1); //1 is My overheads MSG_TYPE
		//status = emberSendReply(reply);
		//emberReleaseMessageBuffer(reply);
		//CheckEmberStatus(status);
	//}
}

//this will ask nick if there are any cmds for Nigel and return the cmd, this also ensures that whilst downloading
//firmware or a task the CU will only deal with one transmitter at a time (i may change this for tasks to x number of transmitters)
boolean SendCmds(int8u *SerialNum, int8u *OutgoingCmd, int8u *CmdStore)//for SENSOR's only
{
	int8u ChkCmdResult;
	unsigned short int Cmd_ID;

	ChkCmdResult = AnyCmdsFor(SerialNum, CmdStore);
	if(ChkCmdResult == ERROR) Error_Note("ERROR RETURNED FROM AnyCmdsFor() **CODE", ChkCmdResult, SerialNum);

	printf("ChkCmdResult is 0x%X\n\r",ChkCmdResult);
	if(ChkCmdResult == YES)
	{
		//this is a debug print
		//printf("AnyCmdsFor returned 0x%X 0x%X 0x%X 0x%X for Sno %s\n\r", CmdStore[0], CmdStore[1], CmdStore[2], CmdStore[3], SerialNum); //debug only

		//New cmd id is two bytes now
		//CmdStore[4]//MSB
		//CmdStore[3]//LSB
		Cmd_ID = CmdStore[3] + (CmdStore[4]<<8);


		if(!HandlingTransmitter)
		{
			//printf("Got into !HandlingTransmitter, Cmd_ID is %i\n\r",Cmd_ID);

			switch(Cmd_ID)	//indicates start of a big download
			{
				case C_NEWTASK:
				case C_ER_PGM :
				case C_EMBER_ER:
					HandlingTransmitter = TRUE;
					//store the serial number
					MEMCOPY(CurrentTransmitter, SerialNum, SIZE_SNUM);
					TaskFirmwareTimer = TRUE;		//timer to ensure it doesn't get locked up
					CheckTaskFirmTimeOut = the_time + TaskFirmTimeOutPeriod; //set up the timer
					printf("Sending first Cmd\n\r");
					//NODE_FIRST_CMD
					OutgoingCmd[0] = NODE_FIRST_CMD;
					MEMCOPY(&OutgoingCmd[1], CmdStore, CmdStore[0] + 1);
					return (TRUE);
			}
			OutgoingCmd[0] = NODE_INCOMMING_CMD;
			MEMCOPY(&OutgoingCmd[1], CmdStore, CmdStore[0] + 1);
			return (TRUE);
		}
		else
		{	//if we are dealing with a transmitter at the moment
			if(String_Compare(SerialNum, 0, CurrentTransmitter, 0, SIZE_SNUM))//is it this one??
			{
				//This is the current transmitter so reset the timer
				CheckTaskFirmTimeOut = the_time + TaskFirmTimeOutPeriod; //gives transmitter best chance of recieving all its cmds before anyone else
																		//gets a go

				switch(Cmd_ID)	//check to see if this is the last cmd for this transmitter
				{
					case C_DONE_NEW_TASK:
					case C_DONE:
					case C_BOOTLD:/*C_DONE_FLASHWR*/
					case C_EMBER_BOOT:
					HandlingTransmitter = FALSE;
					//reset the serial number
					MEMSET(CurrentTransmitter, 0, SIZE_SNUM);
					TaskFirmwareTimer = FALSE;		//timer to ensure it doesn't get locked up
					printf("Sending last Cmd\n\r");
					OutgoingCmd[0] = NODE_LAST_CMD;
					MEMCOPY(&OutgoingCmd[1], CmdStore, CmdStore[0] + 1);
					return (TRUE);
				}
				OutgoingCmd[0] = NODE_INCOMMING_CMD;
				MEMCOPY(&OutgoingCmd[1], CmdStore, CmdStore[0] + 1);
				printf("Sending other Cmds\n\r");
				return (TRUE);
			}
			else//means i have cmds for this transmitter but it will have to wait until i finish with the current device
			{
				//this is where i will lock out other transmitters
				OutgoingCmd[0] = NODE_CMD_WAITING;
				MEMSET(CmdStore, 0, 4);	//just to make sure there is nothing to send
				return (TRUE);
			}
		}
	}
	else return (FALSE);//there are no cmds for this transmitter

}

void DataPacket1(int8u *contents, int8u Device, int8u type)
{
	int8u DataSize = 0;
	int8u SequenceNumber;
	int8u BoundaryCheck;
	int8u SeqnumCalc;

	switch(type)
	{

		case SENSOR:
			SequenceNumber = contents[SEQNUMLOC];
			if(SensorSNid[Device].GotSequence) { //need to check if this is a reflection as may be part of an old MB
				BoundaryCheck = SensorSNid[Device].Pkt1SeqNum - 4;
				if(SequenceNumber == 44) {
					SeqnumCalc = 252;
					if(SeqnumCalc == BoundaryCheck) {
						Duplicate = TRUE;					//this will look one packet sequence block in the past
						Error_Note("BOUNDARY Reflection Packet 1 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
						Error_Note("BOUNDARY Currently Expecting Packet 1 Sequence Number :", SensorSNid[Device].Pkt1SeqNum, SensorSNid[Device].SerialNumber);
						return;
					}
				}

			}

			if((SensorSNid[Device].GotSequence) && (SequenceNumber == (SensorSNid[Device].Pkt1SeqNum-4))){ //need to check if this is a reflection as may be part of an old MB
				Duplicate = TRUE;					//this will look one packet sequence block in the past
				Error_Note("Reflection Packet 1 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Error_Note("Currently Expecting Packet 1 Sequence Number :", SensorSNid[Device].Pkt1SeqNum, SensorSNid[Device].SerialNumber);
				return;
			}

			if(SensorSNid[Device].GotSequence && SensorSNid[Device].Pkt1SeqNum != SequenceNumber) {
				//printf("!!----------------Unexpected Packet 1 Sequence Number----------------------!!\n\r");		//debug only
				Error_Note("Unexpected Packet 1 Sequence Number:", SensorSNid[Device].Pkt1SeqNum, SensorSNid[Device].SerialNumber); //writes to a file
				Error_Note("Been Sent Packet 1 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Print_Packet_Time(SensorSNid[Device].LastPacketRecieved);
				Error_Note("Will now quit", 0, "N/A");
				BadPacketNumber = TRUE;
				exit(0);
				//return;

			}
			if((!SensorSNid[Device].GotSequence) && (SensorSNid[Device].Pkt1SeqNum == SequenceNumber))
			{
				//printf("\n\r!!!!!!!Duplicate Packet Ignor 1!!!!!!!!!!!!\n\r");		//debug only
				Duplicate = TRUE;
			}
			else
			{	//lets finally deal with some data
				SensorSNid[Device].LastPacketRecieved = the_time + DataPacketTimeOut;	  //reset the timout
				if(!SensorSNid[Device].GotSequence){
					GetSequence(Device, 1, SequenceNumber);
				}
				DataSize = contents[BLOCK_SIZE_LOC];
				if(DataSize<=LESSTHN1PKT)		SensorSNid[Device].RequiredPackets = 1;  //= added 19-02-08 buggers up if 66Byte pkt is
				if(DataSize>MORETHN1PKT)		SensorSNid[Device].RequiredPackets = 2;  //rx'd reports it as needing 2 pkts
				if(DataSize>MORETHN2PKT)		SensorSNid[Device].RequiredPackets = 3;
				if(DataSize>MORETHN3PKT)		SensorSNid[Device].RequiredPackets = 4;
				SensorSNid[Device].Data[BLKLEN_LOC] = DataSize;
				MEMCOPY((SensorSNid[Device].Data + PKT1START), (contents + DATAPKT1STRT_LOC), PKT1_DATASIZE);
				SensorSNid[Device].Packet1 = 1;
				//fprintf(stdout, "Packets required is %i\n\r",SensorSNid[Device].RequiredPackets);		//debug only
//				printf("Got Packet 1 Sequence Num %i:Packets required is %i\n\r", SequenceNumber, SensorSNid[Device].RequiredPackets);		//debug only
			}
			break;

		case MOBILE:
			DataSize = contents[BLOCK_SIZE_LOC];
			if(DataSize<=LESSTHN1PKT)		MobileSNid[Device].RequiredPackets = 1;
			if(DataSize<MORETHN1PKT)		MobileSNid[Device].RequiredPackets = 2;
			if(DataSize<MORETHN2PKT)		MobileSNid[Device].RequiredPackets = 3;
			if(DataSize>MORETHN3PKT)		MobileSNid[Device].RequiredPackets = 4;
			MobileSNid[Device].Data[BLKLEN_LOC] = DataSize;
			MEMCOPY((MobileSNid[Device].Data + PKT1START), (contents + DATAPKT1STRT_LOC), PKT1_DATASIZE);
			MobileSNid[Device].Packet1 = 1;
			break;

		case HANDHELD:
			DataSize = contents[BLOCK_SIZE_LOC];
			if(DataSize<=LESSTHN1PKT)		HandheldSNid[Device].RequiredPackets = 1;
			if(DataSize<MORETHN1PKT)		HandheldSNid[Device].RequiredPackets = 2;
			if(DataSize<MORETHN2PKT)		HandheldSNid[Device].RequiredPackets = 3;
			if(DataSize>MORETHN3PKT)		HandheldSNid[Device].RequiredPackets = 4;
			HandheldSNid[Device].Data[BLKLEN_LOC] = DataSize;
			MEMCOPY((HandheldSNid[Device].Data + PKT1START), (contents + DATAPKT1STRT_LOC), PKT1_DATASIZE);
			HandheldSNid[Device].Packet1 = 1;
			break;

	}

}


void DataPacket2(int8u *contents, int8u Device, int8u type)
{
	int8u SequenceNumber;
	int8u BoundaryCheck;
	int8u SeqnumCalc;

	switch(type)
	{

		case SENSOR:
			SequenceNumber = contents[SEQNUMLOC];
			if(SensorSNid[Device].GotSequence) { //need to check if this is a reflection as may be part of an old MB
				BoundaryCheck = SensorSNid[Device].Pkt2SeqNum - 4;
				if(SequenceNumber == 45) {
					SeqnumCalc = 253;
					if(SeqnumCalc == BoundaryCheck) {
						Duplicate = TRUE;					//this will look one packet sequence block in the past
						Error_Note("BOUNDARY Reflection Packet 2 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
						Error_Note("BOUNDARY Currently Expecting Packet 2 Sequence Number :", SensorSNid[Device].Pkt2SeqNum, SensorSNid[Device].SerialNumber);
						return;
					}
				}

			}

			if((SensorSNid[Device].GotSequence) && (SequenceNumber == (SensorSNid[Device].Pkt2SeqNum-4))){ //need to check if this is a reflection as may be part of an old MB
				Duplicate = TRUE;					//this will look one packet sequence block in the past
				Error_Note("Reflection Packet 2 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Error_Note("Currently Expecting Packet 2 Sequence Number :", SensorSNid[Device].Pkt2SeqNum, SensorSNid[Device].SerialNumber);
				return;
			}

			if(SensorSNid[Device].GotSequence && SensorSNid[Device].Pkt2SeqNum != SequenceNumber) {
				//printf("!!----------------Unexpected Packet 2 Sequence Number----------------------!!\n\r");		//debug only
				Error_Note("Unexpected Packet 2 Sequence Number:", SensorSNid[Device].Pkt2SeqNum, SensorSNid[Device].SerialNumber); //writes to a file
				Error_Note("Been Sent Packet 2 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Print_Packet_Time(SensorSNid[Device].LastPacketRecieved);
				Error_Note("Will now quit", 0, "N/A");
				BadPacketNumber = TRUE;
				exit(0);
				//return;

			}
			if((!SensorSNid[Device].GotSequence) && (SensorSNid[Device].Pkt2SeqNum == SequenceNumber))
			{
				//printf("\n\r!!!!!!!!Duplicate Packet Ignor 2!!!!!!!!!!\n\r");		//debug only
				Duplicate = TRUE;
			}
			else
			{	//lets finally deal with some data
				SensorSNid[Device].LastPacketRecieved = the_time + DataPacketTimeOut;	  //reset the timout
				if(!SensorSNid[Device].GotSequence){
					GetSequence(Device, 2, SequenceNumber);
				}
				MEMCOPY((SensorSNid[Device].Data + PKT2START), (contents + DATASTRT_LOC), PKT2_DATASIZE);
				SensorSNid[Device].Packet2 = 1;
				//fprintf(stdout, "Got Pack 2\n\r");		//debug only
//				printf("Got Pack 2 Sequence Num %i:\n\r", SequenceNumber);		//debug only
			}
			break;

		case MOBILE:
			MEMCOPY((MobileSNid[Device].Data + PKT2START), (contents + DATASTRT_LOC), PKT2_DATASIZE);
			MobileSNid[Device].Packet2 = 1;
			break;

		case HANDHELD:
			MEMCOPY((HandheldSNid[Device].Data + PKT2START), (contents + DATASTRT_LOC), PKT2_DATASIZE);
			HandheldSNid[Device].Packet2 = 1;
			break;

	}

}

void DataPacket3(int8u *contents, int8u Device, int8u type)
{
	int8u SequenceNumber;
	int8u BoundaryCheck;
	int8u SeqnumCalc;

	switch(type)
	{

		case SENSOR:
			SequenceNumber = contents[SEQNUMLOC];
			if(SensorSNid[Device].GotSequence) { //need to check if this is a reflection as may be part of an old MB
				BoundaryCheck = SensorSNid[Device].Pkt3SeqNum - 4;
				if(SequenceNumber == 46) {
					SeqnumCalc = 254;
					if(SeqnumCalc == BoundaryCheck) {
						Duplicate = TRUE;					//this will look one packet sequence block in the past
						Error_Note("BOUNDARY Reflection Packet 3 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
						Error_Note("BOUNDARY Currently Expecting Packet 3 Sequence Number :", SensorSNid[Device].Pkt3SeqNum, SensorSNid[Device].SerialNumber);
						return;
					}
				}

			}

			if((SensorSNid[Device].GotSequence) && (SequenceNumber == (SensorSNid[Device].Pkt3SeqNum-4))){ //need to check if this is a reflection as may be part of an old MB
				Duplicate = TRUE;					//this will look one packet sequence block in the past
				Error_Note("Reflection Packet 3 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Error_Note("Currently Expecting Packet 3 Sequence Number :", SensorSNid[Device].Pkt3SeqNum, SensorSNid[Device].SerialNumber);
				return;
			}

			if(SensorSNid[Device].GotSequence && SensorSNid[Device].Pkt3SeqNum != SequenceNumber) {
				//printf("!!----------------Unexpected Packet 3 Sequence Number----------------------!!\n\r");		//debug only
				Error_Note("Unexpected Packet 3 Sequence Number:", SensorSNid[Device].Pkt3SeqNum, SensorSNid[Device].SerialNumber); //writes to a file
				Error_Note("Been Sent Packet 3 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Print_Packet_Time(SensorSNid[Device].LastPacketRecieved);
				BadPacketNumber = TRUE;
				Error_Note("Will now quit", 0, "N/A");
				exit(0);
				//return;

			}
			if((!SensorSNid[Device].GotSequence) && (SensorSNid[Device].Pkt3SeqNum == SequenceNumber))
			{
				//printf("\n\r!!!!!!!!!Duplicate Packet Ignor 3!!!!!!!!!!!!\n\r");		//debug only
				Duplicate = TRUE;
			}
			else
			{	//lets finally deal with some data
				SensorSNid[Device].LastPacketRecieved = the_time + DataPacketTimeOut;	  //reset the timout
				if(!SensorSNid[Device].GotSequence){
					GetSequence(Device, 3, SequenceNumber);
				}

				MEMCOPY((SensorSNid[Device].Data + PKT3START), (contents + DATASTRT_LOC), PKT3_DATASIZE);
				SensorSNid[Device].Packet3 = 1;
				//fprintf(stdout, "Got Pack 3\n\r");		//debug only
//				printf("Got Pack 3 Sequence Num %i:\n\r", SequenceNumber);		//debug only
			}
			break;

		case MOBILE:
			MEMCOPY((MobileSNid[Device].Data + PKT3START), (contents + DATASTRT_LOC), PKT3_DATASIZE);
			MobileSNid[Device].Packet3 = 1;
			break;

		case HANDHELD:
			MEMCOPY((HandheldSNid[Device].Data + PKT3START), (contents + DATASTRT_LOC), PKT3_DATASIZE);
			HandheldSNid[Device].Packet3 = 1;
			break;

	}

}

void DataPacket4(int8u *contents, int8u Device, int8u type)
{
	int8u SequenceNumber;
	int8u BoundaryCheck;
	int8u SeqnumCalc;

	switch(type)
	{

		case SENSOR:
			SequenceNumber = contents[SEQNUMLOC];
			if(SensorSNid[Device].GotSequence) { //need to check if this is a reflection as may be part of an old MB
				BoundaryCheck = SensorSNid[Device].Pkt4SeqNum - 4;
				if(SequenceNumber == 47) {
					SeqnumCalc = 255;
					if(SeqnumCalc == BoundaryCheck) {
						Duplicate = TRUE;					//this will look one packet sequence block in the past
						Error_Note("BOUNDARY Reflection Packet 4 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
						Error_Note("BOUNDARY Currently Expecting Packet 4 Sequence Number :", SensorSNid[Device].Pkt4SeqNum, SensorSNid[Device].SerialNumber);
						return;
					}
				}

			}

			if((SensorSNid[Device].GotSequence) && (SequenceNumber == (SensorSNid[Device].Pkt4SeqNum-4))){ //need to check if this is a reflection as may be part of an old MB
				Duplicate = TRUE;					//this will look one packet sequence block in the past
				Error_Note("Reflection Packet 4 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Error_Note("Currently Expecting Packet 4 Sequence Number :", SensorSNid[Device].Pkt4SeqNum, SensorSNid[Device].SerialNumber);
				return;
			}

			if(SensorSNid[Device].GotSequence && SensorSNid[Device].Pkt4SeqNum != SequenceNumber) {
				//printf("!!----------------Unexpected Packet 4 Sequence Number----------------------!!\n\r");		//debug only
				Error_Note("Unexpected Packet 4 Sequence Number:", SensorSNid[Device].Pkt4SeqNum, SensorSNid[Device].SerialNumber); //writes to a file
				Error_Note("Been Sent Packet 4 Sequence Number :", SequenceNumber, SensorSNid[Device].SerialNumber);
				Print_Packet_Time(SensorSNid[Device].LastPacketRecieved);
				BadPacketNumber = TRUE;
				Error_Note("Will now quit", 0, "N/A");
				exit(0);
				//return;

			}
			if((!SensorSNid[Device].GotSequence) && (SensorSNid[Device].Pkt4SeqNum == SequenceNumber))
			{
				//printf("\n\r!!!!!!!!!!!Duplicate Packet Ignor 4!!!!!!!!!!!!!\n\r");		//debug only
				Duplicate = TRUE;
			}
			else
			{	//lets finally deal with some data
				SensorSNid[Device].LastPacketRecieved = the_time + DataPacketTimeOut;	  //reset the timout
				if(!SensorSNid[Device].GotSequence){
					GetSequence(Device, 4, SequenceNumber);
				}

				MEMCOPY((SensorSNid[Device].Data + PKT4START), (contents + DATASTRT_LOC), PKT4_DATASIZE);
				SensorSNid[Device].Packet4 = 1;
				//fprintf(stdout, "Got Pack 4\n\r");		//debug only
//				printf("Got Pack 4 Sequence Num %i:\n\r", SequenceNumber);		//debug only
			}
			break;

		case MOBILE:
			MEMCOPY((MobileSNid[Device].Data + PKT4START), (contents + DATASTRT_LOC), PKT4_DATASIZE);
			MobileSNid[Device].Packet4 = 1;
			break;

		case HANDHELD:
			MEMCOPY((HandheldSNid[Device].Data + PKT4START), (contents + DATASTRT_LOC), PKT4_DATASIZE);
			HandheldSNid[Device].Packet4 = 1;
			break;

	}

}


void GetSequence(int8u Device, int8u PacketNumber, int8u SequenceNumber)
{
	int8u FinalValue = 0;

	if(PacketNumber == 1) FinalValue = SequenceNumber + 4;
	if(PacketNumber == 2) FinalValue = SequenceNumber + 3;
	if(PacketNumber == 3) FinalValue = SequenceNumber + 2;
	if(PacketNumber == 4) FinalValue = SequenceNumber + 1;

	SensorSNid[Device].Pkt1SeqNum = FinalValue - 4;
	SensorSNid[Device].Pkt2SeqNum = FinalValue - 3;
	SensorSNid[Device].Pkt3SeqNum = FinalValue - 2;
	SensorSNid[Device].Pkt4SeqNum = FinalValue - 1;
	SensorSNid[Device].GotSequence = TRUE;
}


void SendToNick(int8u Device)		//not used anymore 20-06-06
{
	int8u Error = 0;
	SensorSNid[Device].Packet1 = 0;
	SensorSNid[Device].Packet2 = 0;
	SensorSNid[Device].Packet3 = 0;
	SensorSNid[Device].Packet4 = 0;
	SensorSNid[Device].RequiredPackets = 0;

	//Send data to Nick
	Error = DoRxCmd(SensorSNid[Device].SerialNumber, SensorSNid[Device].Data);
	if(Error != NOERROR)
	{
		Error_Note("ERROR RETURNED FROM DoRxCmd() **CODE", Error, SensorSNid[Device].SerialNumber);
		//fprintf(stdout, "Data Save Error\n\r");
	}
	//else
	//{
		//MEMSET(SensorSNid[Device].Data,0,MAX_DATASIZE);  //clear down the data
	//}
	SensorSNid[Device].GotMonsterBlock = FALSE;
}


void PrintData_file(int8u Device)		//debug stuff only
{
	FILE *info;
	int8u i;

	info = fopen("/home/ember/Ember/Data_file.txt" , "ab+");
	fprintf(info,"Serial Num is:%s\r\n",SensorSNid[Device].SerialNumber);

	fprintf(info,"Packet size is (decimal): %d\r\n",SensorSNid[Device].Data[0]);

	for(i=0;i<(SensorSNid[Device].Data[0]);i++)
	{
		fprintf(info,"%02X,",SensorSNid[Device].Data[i]);
	}

	fprintf(info,"\n\r");

	fclose(info);
}




