/*
This files contains the code that manages the network, sends advertisments/ network update signals and
deals with the incoming responces to those signals

*/

static void RegisterDeviceWithCU(int8u *contents, int8u DevType);
static void sendAdvertisement(void);
static void networkUpdate(void);
void networkUpdateResponce (int8u *contents);
int8u CheckBindingTable(int8u *contents);
void LoadParamStruct(int8u *contents);

boolean SendAd = FALSE;


void CheckEmberStatus(EmberStatus status) //only called from an EmberSendxxxx eg datagram/multicast etc
{
	switch(status)
	{
		case EMBER_SUCCESS: //means everything is ok
		  break;

		case EMBER_NETWORK_DOWN:
		  Error_Note("Tired to transmit but emberstatus returned !EMBER_NETWORK_DOWN!", status, "N/A");
		  exit(0);

		case EMBER_MAX_MESSAGE_LIMIT_REACHED:  //not a crash condition but does sometimes happen don't really know what to do about it yet
		  break;

		default:
		  //Error_Note("emberstatus returned:", status, "N/A"); //debug only
		  break;
	}
}

static void sendAdvertisement(void)
{
  EmberStatus status;
  EmberMessageBuffer buffer = 0;

  // the data - msg type, and sink address
  globalBuffer[0] = CU_ADVERTISEMENT;
  MEMCOPY(&(globalBuffer[1]), emberGetEui64(), EUI_LEN);

  //fprintf(stdout, "Sending %x\r\n", status);

  // copy the data into a packet buffer
  buffer = emberFillLinkedBuffers(globalBuffer, EUI_LEN+1);

  // check to make sure a buffer is available
  if (buffer == EMBER_NULL_MESSAGE_BUFFER)
  {
    printf("TX ERROR [sink advertise], OUT OF BUFFERS\r\n");
    Error_Note("TX ERROR [sink advertise], OUT OF BUFFERS\r\n", 0, "N/A");
	return;
  }

  // send the message
  status = emberSendMulticast(MULTICAST_BINDING_INDEX, buffer);
  // done with the packet buffer
  emberReleaseMessageBuffer(buffer);
  CheckEmberStatus(status);

  printf("TX [sink advertise], status is 0x%x\r\n", status);
}

static void RegisterDeviceWithCU(int8u *contents, int8u DevType)
{
	EmberBindingTableEntry binding;
    int8u availBinding, Error;
    int8u ExsistingDevice;
	int8u PassToNick[80];			//buffer containing commision structure for nick
	int8u EuiToNick[30];
	int8u Device;
	EmberStatus status;
	EmberMessageBuffer buffer = 0;
	//int i;

	//retrieve information from the packet and load up my structure
	LoadParamStruct(contents);
	//first check to see if an entry has already been allocated
    //for this meter
	ExsistingDevice = CheckBindingTable(contents);

	//Information to pass to nick------------------------
	MEMCOPY(PassToNick, contents + 9, SIZE_NICKSTRUCT);
	//PassToNick[68] = 'x';
	//PassToNick[69] = 'X';
	//PassToNick[70] = 'x';
	sprintf(EuiToNick, "%02X%02X%02X%02X%02X%02X%02X%02X",contents[8], contents[7], contents[6],contents[5], contents[4], contents[3], contents[2], contents[1]);
	//printf("EUI IS:%02X%02X%02X%02X%02X%02X%02X%02X \n\r",contents[1], contents[2], contents[3],contents[4], contents[5], contents[6], contents[7], contents[8]);

	//for(i=0;i<71;i++) printf("%c",PassToNick[i]);
	//exit(0);


	//MEMCOPY(EuiToNick, contents + 1, SIZE_EUI64);
	//int8u EuiToNick[9] = 0;
	//---------------------------------------------------

	//retrieve information from the packet and load up my structure
	//LoadParamStruct(contents);


	if(!ExsistingDevice)	//Picked up a new sensor
	{
		if(ChckDeviceSN(contents,DevType))
		{
			binding.type = EMBER_UNICAST_BINDING;
			MEMCOPY(binding.identifier, contents + 1, 8);
			binding.local = ourTerminal;
			binding.remote = ourTerminal;

			availBinding = findAvailableBinding(contents+1);
			if ((availBinding != 0xFF) && (emberSetReplyBinding(availBinding, &binding) == EMBER_SUCCESS))
			{
				switch(DevType)		//will load the binding table address here save time later
				{
					case SENSOR:
						SensorSNid[CurrentSensor].BindingTableIndex = availBinding;
						break;

					case MOBILE:
						MobileSNid[CurrentSensor].BindingTableIndex = availBinding;
						break;

					case HANDHELD:
						HandheldSNid[CurrentSensor].BindingTableIndex = availBinding;
						break;
				}

			}
			else
			{
				printf("Could not set binding entry.\n");
				Error_Note("Could not set binding entry.\r\n", 0, "N/A");
				return;
			}

			globalBuffer[0] = CU_OKBIND;
			MEMCOPY(&(globalBuffer[1]), emberGetEui64(), EUI_LEN);   //CU EUI Number
			globalBuffer[9] = SensorSNid[CurrentSensor].AbnormalGatewayShutdown;  //more cipher lite fun and games
			buffer = emberFillLinkedBuffers(globalBuffer, EUI_LEN+2);
			//tell node its got a CU
			status = emberSendReply(buffer);
			emberReleaseMessageBuffer(buffer);
			CheckEmberStatus(status);
			Error = foundSno(PassToNick,EuiToNick);		//Send Nick info
			if(Error == NOERROR)
			{
				printf("Snum found %s\n\r", Cipher_Params_CU.SerialNum); //Send Nick info
			}
			else Error_Note("ERROR RETURNED FROM foundSno() **CODE", Error, Cipher_Params_CU.SerialNum);
		}
		else printf("Thats Not one of mine %s\n\r", Cipher_Params_CU.SerialNum);
	}
	else
	{   //need to update device information here mesher etc
		//now needed for last heard stuff
		Device = SearchDeviceArray(Cipher_Params_CU.DeviceType);//update Serial number array
		if(Device != 254) UpdateDevice(Device, Cipher_Params_CU.DeviceType);
		Error = foundSno(PassToNick,EuiToNick);		//Send Nick info
	}
}


static void networkUpdate(void)
{
  EmberStatus status;
  EmberMessageBuffer buffer = 0;

  // the data - msg type, and sink address
  globalBuffer[0] = NETWORK_UPDATE;
  MEMCOPY(&(globalBuffer[1]), emberGetEui64(), EUI_LEN);

  // copy the data into a packet buffer
  buffer = emberFillLinkedBuffers(globalBuffer, EUI_LEN+1);

  // check to make sure a buffer is available
  if (buffer == EMBER_NULL_MESSAGE_BUFFER)
  {
    printf("TX ERROR [Update Signal], OUT OF BUFFERS\r\n");
    Error_Note("TX ERROR [Update Signal], OUT OF BUFFERS\r\n", 0, "N/A");
	return;
  }

  // send the message
  status = emberSendMulticast(MULTICAST_BINDING_INDEX, buffer);
  // done with the packet buffer
  emberReleaseMessageBuffer(buffer);
  CheckEmberStatus(status);



  printf("Sent Update Signal\r\n");

}

void networkUpdateResponce (int8u *contents)
{
    EmberBindingTableEntry binding;
	int8u i, Device;

	//first check to see if this is one of our nodes
    for (i = 0; i < MAX_METERS; i++)
	{
      emberGetBinding(i, &binding);
      if (MEMCOMPARE(binding.identifier, (contents+1), 8) == 0)
	  {
		//we should note the senders binding to update the cost and route
		//to the meter.
	    emberNoteSendersBinding(i);
		LoadParamStruct(contents);
		Device = SearchDeviceArray(Cipher_Params_CU.DeviceType);//update Serial number array
		if(Device != 254) UpdateDevice(Device, Cipher_Params_CU.DeviceType);
		else {
			printf("Error Updating signal information\r\n"); 	//should not get here. debug info
			Error_Note("Error Updating signal information\r\n", 0, "N/A");
			return;
		}

		printf("Updating signal information\r\n");
	    return;
      }
    }
}


int8u CheckBindingTable(int8u *contents)
{
	int8u i;
	EmberBindingTableEntry binding;
    EmberStatus status;
	EmberMessageBuffer buffer = 0;
	int8u DeviceLocation = 0;

	for (i = 0; i < MAX_METERS; i++) {
		emberGetBinding(i, &binding);
		if (MEMCOMPARE(binding.identifier, (contents+1), 8) == 0) {
			//we should note the device binding to update the cost and route
			emberNoteSendersBinding(i);	//indicates we already have this device but it has disconected and now re-connected
			DeviceLocation = SearchDeviceArrayForRemoved(SENSOR);
			if(DeviceLocation != 254){   //need to clear down the monster block and sequence malarky
				//SensorSNid[DeviceLocation].LastPacketRecieved = the_time + DataPacketTimeOut;
				SensorSNid[i].LastPacketRecieved = the_time - DataPacketTimeOut; //just to initialise
				SensorSNid[DeviceLocation].Pkt1SeqNum = 255;
				SensorSNid[DeviceLocation].Pkt2SeqNum = 255;
				SensorSNid[DeviceLocation].Pkt3SeqNum = 255;
				SensorSNid[DeviceLocation].Pkt4SeqNum = 255;
				SensorSNid[DeviceLocation].GotSequence = FALSE;
			}
			globalBuffer[0] = CU_OKBIND;
			MEMCOPY(&(globalBuffer[1]), emberGetEui64(), EUI_LEN);   //CU EUI Number
			buffer = emberFillLinkedBuffers(globalBuffer, EUI_LEN+1);
			//tell node its got a CU
			status = emberSendReply(buffer);
			emberReleaseMessageBuffer(buffer);
			CheckEmberStatus(status);

			return(TRUE);  //this indicates that this device already has a binding table entry on the CU
		}					//so it probably went out of range for a while and lost its CU and started looking again
    }

	return(FALSE);		//this means we do not have a binding table entry for the device and it should be treated as a
}						//new device


//find an available binding in the binding table
int8u findAvailableBinding(int8u *remoteEui64)
{
  EmberBindingTableEntry binding;
  int8u i;

  for (i = 0; i < MAX_METERS; i++) {
    emberGetBinding(i, &binding);
    if (binding.type == EMBER_UNUSED_BINDING) {
      return i;
    }
  }
  return 0xFF;
}


void LoadParamStruct(int8u *contents)
{
	Cipher_Params_CU.Model = contents[AD_MODCLS];
	Cipher_Params_CU.DeviceType = contents[AD_RFMODCLS];
	Cipher_Params_CU.bMesher = contents[AD_DEVCLS];
	MEMCOPY((Cipher_Params_CU.SerialNum),(contents+AD_SNUM), SIZE_SNUM+1);
	MEMCOPY((Cipher_Params_CU.ModelStr),(contents+AD_MDLSTR), SIZE_MDLSTR+1);
	MEMCOPY((Cipher_Params_CU.FirmwareStr),(contents+AD_FRMSTR), SIZE_FRMSTR+1);
	Cipher_Params_CU.GwCmdVn = contents[AD_CMDVN];
	Cipher_Params_CU.GwDataVn = contents[AD_DATAVN];
	MEMCOPY((Cipher_Params_CU.Padding ),(contents+AD_PADDING), SIZE_PADDING);
	MEMCOPY((Cipher_Params_CU.RFFirmwareStr),(contents+AD_RFFRMSTR), SIZE_RFRFMSTR+1);
	//The above is the same as the node stuff and Nick stuff the next bit is mine only
	MEMCOPY((Cipher_Params_CU.Eui64),(contents+EUI64_START), SIZE_EUI64+1);

	//printf("\n\rSnum %s\n\r",Cipher_Params_CU.SerialNum);
	//printf("\n\rModel %s\n\r",Cipher_Params_CU.ModelStr);
	//printf("\n\rNW Firmware %s\n\r",Cipher_Params_CU.FirmwareStr);
	//printf("\n\rPM Frimware %s\n\r",Cipher_Params_CU.RFFirmwareStr);
}








