/**
   @file cipher-aggregator.c
   @description This is the main code for the aggregator portion of
   the amr sample application. This code is written with the intention
   that it can compiled for either a node or a host. For a more comprehensive
   description of the use of this application see amr.h. Comments about the
   specific features of the aggregator are provided inline.

    @see cipher-rf.h

   Copyright 2004 by Ember Corporation. All rights reserved.              *80*
 */


/**** NODE / HOST DIFFERENCE ****
  Many of the differences between the node and the host are abstracted
  for the developer by Ember's host-framework module referred to as the
  "host-api". The host-api is included below. For a more comprehensive
  description of how it works see app/host-framework/c/host-api.h

 @see app/host-framework/c/host-api.h

*/
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <ctype.h>
//#include "/usr/include/cipher/common.h"
//#include <cipher/common.h>
#include <cipher/emberF.h>
//#include <cipher/error_codes.h>		//dont need if include cipher_defines
#include <cipher/cipher_defines.h>
#include "app/host-framework/c/host-api.h"
#include "app/host-framework/c/packet-buffer.h"
#include "app/host-framework/c/util/command-interpreter.h"
#include "app/Cipher/app-timers.h"
#include "stack/include/error.h"
#include "app/Cipher/rnap/TimeStuff.h"
#include "app/Cipher/rnap/cipher-rf.h"
//#include "app/Cipher/rnap/Collection-Unit-Error-Handling.h"
#include "app/Cipher/rnap/cipher-aggregator.h"
#include "app/Cipher/rnap/Packet-Structure.h"
#include "app/Cipher/rnap/Device-management.h"
#include "app/Cipher/rnap/Network-comish.h"
#include "app/Cipher/rnap/Collection-Unit-Error-Handling.h"
#include "app/Cipher/rnap/Data-packets.h"
#include "app/Cipher/rnap/Collection-Unit_cmds.h"
#include "app/Cipher/rnap/cipher-aggregator-cmds.c"

//------------------------------------------------------------------------------
//a flag to track when the network is up or down
boolean networkUp = FALSE;
boolean NetworkHasBeenUp = FALSE;
//time_t the_time;

//Endpoints only need to be defined for the node, not for the host.
static EmberInterfaceId PGM serverInterface = {
  AD_HOC_INTERFACE,
  AGGREGATION_SERVER_ENDPOINT,
  4, 'A', 'M', 'R', 0
};

static EmberEndpointDescription PGM endpointDescription = {
  { 0, &serverInterface }
};

int8u emberEndpointCount = 1;
EmberEndpoint emberEndpoints[1] = {
  { 1, &endpointDescription }
};

PGM_NO_CONST PGM_P applicationString = "Aggregator";



int8u NewSno[12];
int8u ReSendLastRadioSession = 0;
int8u comd[100];  //hack
int8u Result;	//useful variable
int8u RnapErrorCount = 0;
boolean ResetYouGit = FALSE;
boolean Quit = FALSE;
boolean HandlingData = FALSE;
boolean WrittenAddress = FALSE;
int i;

//the id of the next meter we expect a report from so we'll know if
//their skipping a node or not.
int8u nextReportId = 0;

//number of meters we know about
static int8u meterCount = 0;



/*
  **** NEW WITH VERSION 3.3 ****
  Before the host application can do anything, it must first associate with
  a network. I have provided a default network parameters struct here which
  is passed into the new emberAssociate() call in the Host API.
*/
static EmberNetworkParameters networkParams =
  {
    0,                  // PAN ID
    TRUE,               // enable relaying
    CIPHER_SAMPLE_POWER,
    CIPHER_SAMPLE_CHANNEL
  };

//commands used by the command interpreter
//extern EmberCommandEntry commands[];
//extern EmberCommandState commandState;

//-----------------------------------------------------------------------------
/* FORWARD DECLARATIONS OF AGGREGATOR SPECIFIC MESSAGE HANDLERS */
static void incomingDatagramHandler(int8u localTerminal, int8u bindingTableIndex,
                                    EmberMessageBuffer message);
static void incomingSequencedHandler(int8u localTerminal, int8u bindingTableIndex,
                                     EmberMessageBuffer message);


void TimedEvents(void);
void CheckListenStatus(void);

//------------------------------------------------------------------------------
/*
  ****NODE / HOST DIFFERENCE****
  On the node, we have to initialize the Ember Stack,
  On the host we need to handle arguments which are passed in
  so that we can initialize the host using the host-api's emberHostInit() which
  takes in two arguments "address and port" so that it can locate the RNAP gateway
  node which proxys its requests on the Ember network.
*/
int main(int argc, char **argv) //host main
{
	fflush(stdout);
	WriteTextFile(VersionNumber);
	InitMyTime();
	//release hog flag
	unlink("/mnt/ram/EmberSlowDownFlag");

	SystemStats.NumberMeshers = 0;
	SystemStats.HeardSomething = FALSE;
	WrittenAddress = FALSE;

	for(Result=0;Result<MAX_SENSORS;Result++) //just make sure i have a clean slate
	{
		memset(&SensorSNid[Result], 0, sizeof(DeviceInformation));
	}

    while(SnoList(NewSno,&ReSendLastRadioSession) == YES)
	{
		printf("Snum is %s.\n\r",NewSno);
		LoadSN(NewSno,ReSendLastRadioSession);
	}

    /* NECESSARY HOST API CALL:
     In order for the Host API to initialize properly the host application must first make a
     call to emberHostInit informing it where to locate the gateway node. The Host API is
     written to connect to a gateway connected to a serial port, a socket or a simulated node
     over stdio.

     1. If two arguments are given at run time, the code below assumes that the host intends to connect
     to an RNAP gateway on a socket where the two arguments are address and port.

     2. If only one argument is given, the Host API assumes that it is connecting to
     a gateway on a serial port where the argument is port.

     3. When no arguments are given, the Host API assumes that it is connecting to
     a simulated gateway over stdio. This connection mode is used internally by Ember
     for simulated testing and development of the Host API and RNAP protocol.

  */
  EmberStatus status;
  if (argc == 3)
    status = emberHostInit(argv[1], atoi(argv[2]), EMBER_HOST_MODE_SOCKET);
  else if (argc == 2){
    status = emberHostInit(argv[1], 0, EMBER_HOST_MODE_SERIAL);
	}
  else {
    fprintf(stdout, "No Gateway location specified, connecting in test mode.\n");
    status = emberHostInit((int8u *)0, 0, EMBER_HOST_MODE_STDIO);
  }
  if (status != 0) {
    fprintf(stdout, "ERROR connecting to gateway, EmberStatus = %d.\n", status);
	Doging = 1;
    return status;
  }

  /* NEW TO VERSION 3.3
    NECESSARY HOST API CALL.
    The application must tell the stack
    that it wants to associate with the network. On the host, this can be done
    at any time, as the host-api will just save the parameter information until the
    gateway node is ready to receive it. The Host API waits until the initial node start
    handshaking is done with the RNAP Gateway before sending the emberAssociate message
    to the gateway.

    NOTE: The networkParams struct is defined in this file above and draws it's default
    settings for channel and power from the definitions in amr.h. If you are using this
    application as a template you will want to be sure and change the default values for
    channel and power in amr.h to your desired settings.
    */
  status = emberAssociate(&networkParams);
	if(status != EMBER_SUCCESS) printf("Error emberAssociate did not work\n\r");
  /* Non-RNAP related module used by this sample application:
     The Command Reader is used by the Host application to read key strokes coming in from
     the user. The commands available are defined in amr-aggregator-cmds.c. You do not need
     to use the command reader to run RNAP it is just a module we wrote for this sample application.
     You are free to use it or discard it if you wish.
  */
  //emberCommandReaderInit(&commandState);

  /* Necessary for this sample app: create multicast binding for meter interaction. Both the
     amr-aggregator and the meters subscribe to a shared multicast binding so that they can
     find eachother when the aggregator sends out multicast advertisements.
  */
  createMulticastBinding(MULTICAST_BINDING_INDEX);

  printf("\n\rVersion %s\n\r",VersionNumber);
  Error_Note("Starting Cipher-host Session Version: ", 0, VersionNumber);
  fflush(stdout);

  //Doging = 11;  now in timestuff.h
  while(TRUE) {

	if(ResetYouGit) //will attempt to reset the rf board via a soft reset not any more creates a shit load of thread's
	{									//this is now more an error reporting tool
		ResetYouGit = FALSE;
		if(RnapErrorCount > 5){
			Error_Note("Too Many RNAP errors will now exit", 0, "N/A");
			WrEmberRunningMessage(1);			//the Nick i'm dead message will indicate a comms breakdown
			exit(0); //
		}

		Error_Note("RNAP Error Lets Go Again", RnapErrorCount, "N/A");
	}

	TimedEvents();

	if(Quit)
	{
		//printf("\n\r!!!!!!! Quit = %x,   HandlingData = %x !!!!!!!!!\n\r", Quit, HandlingData);
		if(!HandlingData){
			Error_Note("Have Been Asked to Quit", 0, "N/A");
			printf("\n\rHave Been Asked to Quit\n\r");
			WrEmberRunningMessage(2);			//the Nick i've quit
			exit(0);
		}
	}


	/*NOTE: This call is used internally by Ember for development. If you are using this
      application as a template for application development you can remove all of the
      references to EMBER_TEST and included code.
    */
    #ifdef EMBER_TEST
      simulatedTimePassesMs(msToNextEvent());
    #endif

    /* Non-RNAP related module used by this sample application:
       This amr sample app uses the timer module included with the sample apps to schedule
       timed events. This module is not part of, and is not used by the RNAP Host API. For
       the sake of this application, timers must be serviced but if you are using this
       app as template and don't want to use the timer module, you do not have
       to service the timers and can remove this call to handleTimers.
    */
    handleTimers();

    /* Non-RNAP related module usde by this sample application:
       As mentioned above with regard to emberCommandReaderInit, this call is part of
       the command reader module included with this sample application. The command reader
       is not used by the Host API so the call below is only necessary if you are using
       the command module.
    */
    //emberProcessCommandInput(&commandState, SERIAL_PORT);

    /*
      This main loop was chewing up 100% of the cpu so to make it relax a bit
      i've added a sleep here.
    */
    //usleep(2); // Old value was 4,
    usleep(1);  //see if Nick notices!!!

	the_time = time((time_t *)0);

	//is it time to send an advertisment
	if(the_time >= Advertisment) //outside of the network message to provoke it!!
	{
		sendAdvertisement();
		Advertisment = the_time + Advertisment_period;
		SendAd = FALSE;
	}

	if(networkUp)
	{
		NetworkUpTimer = the_time + NetworkUpPeriod;
		NetworkHasBeenUp = TRUE;


		//or
		if(SendAd)		//a forced send ad by a leaf node
		{
			sendAdvertisement();
			Advertisment = the_time + Advertisment_period;	//reset the timer
			SendAd = FALSE;
		}

		if(the_time >= Network_update)
		{
			networkUpdate();
			Network_update = the_time + Network_update_period;
		}

		if(!WrittenAddress)  //write the gateways eui64 address to file
		{
			WriteRadioAddress();
			WrittenAddress = TRUE;
		}


        CheckListenStatus(); //trap to try and detect the gone deaf bug


	}



	if(AnyTxMessage() == YES){			//will load up a new serial number passed form nick

		do
		{
			Result = AnySnoRemoved(NewSno);
			if(Result == YES){
				//printf("Remove Snum %s.\n\r",NewSno);		//debug
				SetforRemoval(NewSno);
			}
		}
		while(Result == YES);
		if(Result == ERROR) {
			Error_Note("ERROR RETURNED FROM AnySnoRemoved() **CODE", Result, "N/A");
		}

		do
		{
			Result = AnyNewSno(NewSno);
			if(Result == YES){
				//printf("Adding Snum is %s.\n\r",NewSno);		//debug
				LoadSN(NewSno, 0);
			}
		}
		while(Result == YES);
		if(Result == ERROR) {
			Error_Note("ERROR RETURNED FROM AnyNewSno() **CODE", Result, "N/A");
		}
	}

	if(SensorRemoved || MobileRemoved || HandheldRemoved) RemoveDevice();	//this will only remove one s/n at a time

 }
  return 0;
}

void TimedEvents(void)
{
	if(the_time >= CheckCmdMsg)
	{
		//printf("Check cmd message\n\r");			//debug
		CheckCmdMsg_Result = AnyCmdMessage();		//sets the flag
		CheckCmdMsg = the_time + CheckCmdMsg_period;
		//if(CheckCmdMsg_Result == YES) printf("Got Cmd Message.\n\r");
		//fflush(stdout);			//this is a mod for nicks new log file malarky
	}

	if(TaskFirmwareTimer)
	{
		if(the_time >= CheckTaskFirmTimeOut)//indicates it has timed out
		{
			HandlingTransmitter = FALSE;
			//reset the serial number
			MEMSET(CurrentTransmitter, 0, SIZE_SNUM);
			TaskFirmwareTimer = FALSE;		//timer to ensure it doesn't get locked up
		}

	}

	if(the_time >= ShouldIQuit)
	{
		//printf("\n\rLooking for quit\n\r");
		if(DoIQuit() == YES)
		{
			Quit = TRUE;
			return;
		}
		else
		{
			ShouldIQuit = the_time + QuitCheckPeriod;
		}
	}

	if((the_time >= NetworkUpTimer) && (!networkUp) && (!NetworkHasBeenUp))
	{
		RnapErrorCount = 6;
		ResetYouGit = TRUE;
		Error_Note("Couldn't get network up within 2 mins", 0, "N/A");
		//NetworkHasBeenUp = TRUE;
	}

	if(the_time >= WatchDogTimer)
	{
		WrEmberRunningMessage(Doging);
		WatchDogTimer = the_time + WatchDogTimePeriod;
		Doging++;
	}

	if(the_time >= LookForDeadTxTimer) //should happen every 2 hours
	{
		//Error_Note("Auto Look For Dead Tx", 0, "N/A");
		CheckDeadRemovedTx();
		LookForDeadTxTimer = the_time + LookForDeadTxPeriod;
	}

	if(the_time >= MesherCounterTimer) //should happen every 28mins
	{
		SystemStats.NumberMeshers = 0;        //will update the SystemStats.NumberMeshers variable here
		for(i=0;i<MAX_SENSORS;i++){
            if(SensorSNid[i].Mesher) SystemStats.NumberMeshers++;      //I have meshers!!!!
		}
        //printf("!!!!!NumberMeshers = %d!!!!!\n\r",SystemStats.NumberMeshers); //debug only
		//Error_Note("!!!!!NumberMeshers = ", SystemStats.NumberMeshers, "N/A");   //debug only
		MesherCounterTimer = the_time + MesherCounterTimerPeriod;
	}

}


//To combat a strange bug of embernet, the Gateway seems to go deaf, it can transmit but seems not to be able to hear anything.
void CheckListenStatus(void)  //considered the transport application but should not be too bad as is only an RF reset.
{
//LastHerdMesherTimer is initialised to 0, once it hears a Network update signal it will be over written with the current time.
//I don't want this to start if it has never had contact from a device.
    if(LastHerdMesherTimer){
        if(SystemStats.NumberMeshers && (the_time >= LastHerdMesherTimer)) {  //I have meshers and its been LastHerdMesherPeriod since i last herd one
            //HAVE I HEARD FROM ANYTHING??
            if(the_time >= LastHerdAnythingTimer){//I have meshers and normal tx's and have not heard anything for LastHerdAnythingPeriod
                //Will note in emberlog and then bug out to let Nick restart me
                Error_Note("ERROR I THINK I'VE GONE DEAF NOT HEARD FROM MESHERS OR ANYTHING NEED RESET **CODE", 0, "N/A");
                exit(0);
            }
        }
    }
//----------------------------------------------------------------------------------------------------------------------------------
//In the unlikely event that the system has no meshers I will base my decision on the last time I heard any radio
//----------------------------------------------------------------------------------------------------------------------------------
    if(!SystemStats.NumberMeshers && SystemStats.HeardSomething){//Just a check to confirm the system has at some point recieved something and i don't have any Meshers
                                                                 //cleared on reset
        if(the_time >= LastHerdAnythingTimer){//I have only leaf tx's and have not heard anything for LastHerdAnythingPeriod
                //Will note in emberlog and then bug out to let Nick restart me
                Error_Note("ERROR I THINK I'VE GONE DEAF NOT HEARD FROM ANYTHING NEED RESET **CODE", 0, "N/A");
                exit(0);
            }
    }

}




//------------------------------------------------------------------------------
// AGGREGATOR SPECIFIC MESSAGE HANDLERS

//handles messages coming in from meters
static void incomingDatagramHandler(int8u localTerminal, int8u bindingTableIndex, EmberMessageBuffer message)
{
  int8u *contents = emberMessageBufferContents(message);
  int8u RecievedDatabuffer[80];		//Biggest data packet RF500 system sends is 77 bytes

  FILE *fp;

  int8u IncomingDataSize = emberMessageBufferLength(message);

  memset(RecievedDatabuffer, 0, sizeof(RecievedDatabuffer));	      //clear it
  //printf("\n\rMessage Buffer Size is %d\n\r", IncomingDataSize);    //debug code

  //set hog flag
  if((fp=fopen("/mnt/ram/EmberSlowDownFlag","w"))!=NULL)
    fclose(fp);

  switch (contents[0]) {

	case SENSOR_ADVERTISEMENT:
		emberCopyFromLinkedBuffers(message, 0, RecievedDatabuffer, IncomingDataSize);
		RegisterDeviceWithCU(RecievedDatabuffer, SENSOR);
		break;

	case MOBILE_ADVERTISEMENT:
		emberCopyFromLinkedBuffers(message, 0, RecievedDatabuffer, IncomingDataSize);
		RegisterDeviceWithCU(RecievedDatabuffer, MOBILE);
		break;

	case HANDHELD_ADVERTISEMENT:
		emberCopyFromLinkedBuffers(message, 0, RecievedDatabuffer, IncomingDataSize);
		RegisterDeviceWithCU(RecievedDatabuffer, HANDHELD);
		break;

	case NETWORK_UPDATE_OK:
        if(SystemStats.NumberMeshers) LastHerdMesherTimer = the_time + LastHerdMesherPeriod; //rx'd an update signal and i have meshers
		emberCopyFromLinkedBuffers(message, 0, RecievedDatabuffer, IncomingDataSize);
		networkUpdateResponce(RecievedDatabuffer);
		break;

	case SENSOR_DATAPACKET_START:
	case SENSOR_DATAPACKET_2:
	case SENSOR_DATAPACKET_3:
	case SENSOR_DATAPACKET_4:
		emberCopyFromLinkedBuffers(message, 0, RecievedDatabuffer, IncomingDataSize); 	//
		IncommingSensorData(RecievedDatabuffer);
		break;

	default:
		//fprintf(stdout, "rx, datagram we don't understand.\n\r");	//Debug
		break;
  } // end switch

  //release hog flag
  unlink("/mnt/ram/EmberSlowDownFlag");

}

//handles sequenced messages coming from the meters
static void incomingSequencedHandler(int8u localTerminal, int8u bindingTableIndex, EmberMessageBuffer message)
{
  int8u msgType;
  emberCopyFromLinkedBuffers(message, 0, &msgType, 1);
}

static void incomingMulticastHandler(int8u localTerminal, int8u bindingTableIndex, EmberMessageBuffer message)
{
	int8u *contents = emberGetLinkedBuffersPointer(message,0);

    switch (contents[0])
	{
		case LEAFNODE_ADVERTISEMENT:
			printf("Recieved advertise message from leaf node\n\r");
			if(ChckLeafSN(contents, SENSOR)){
                	SendAd = TRUE;//if its one of ours respond
                	printf("and its mine\n\r");
			}
			break;

		default:
			//fprintf(stdout, "rx, multicast we don't understand.\n\r"); 	//Debug
			break;
	}
}


//------------------------------------------------------------------------------
// Host Api Callbacks
// handle incoming messages from the host-api for a more complete explanation
// @see stack/include/ember.h
void emberBeaconLossCallback(void) {

}

void temporaryIncomingBeaconHandler(void) {

}

void emberIncomingMessageHandler(EmberIncomingMessageType type, int8u localTerminal, int8u bindingTableIndex, EmberMessageBuffer message)
{
  int8u *data = emberMessageBufferContents(message);

  if (!emberMessageBufferLength(message))
    return;

   //is this the first time i have recieved anything over the radio
   if(!SystemStats.HeardSomething) SystemStats.HeardSomething = TRUE;
   LastHerdAnythingTimer = the_time + LastHerdAnythingPeriod;

   HandlingData = TRUE; 	//this is the place that i must quit it guarentee's no data being handled
   RnapErrorCount = 0;		//means i am still hearing the RF board
   if(!networkUp) {Error_Note("networkUp has been set false", 0, "N/A"); networkUp = TRUE;} 	//if i get here then surely the network is up

  switch (type) {
  case EMBER_INCOMING_DATAGRAM_REPLY:
    //fprintf(stdout, "host rx incoming datagram reply\n");	//Debug
  case EMBER_INCOMING_DATAGRAM:
    incomingDatagramHandler(localTerminal, bindingTableIndex, message);
    HandlingData = FALSE;
	break;

  case EMBER_INCOMING_SEQUENCED:
    incomingSequencedHandler(localTerminal, bindingTableIndex, message);
    HandlingData = FALSE;
	break;

  case EMBER_INCOMING_MULTICAST:
    incomingMulticastHandler(localTerminal, bindingTableIndex, message);
	HandlingData = FALSE;
	break;

  case EMBER_INCOMING_SHARED_MULTICAST:
  case EMBER_INCOMING_MULTICAST_LOOPBACK:
  default:
    //fprintf(stdout, "I Smell Another Ember Path to profit System Nearby!!!!!!!!!!!!!!!!!!\r\n");  //Debug
    HandlingData = FALSE;
	break;
  }
}

// Called when a datagram has been transmitted. The status indicates
// the success or failure of the transmission
void emberMessageSent(int8u bindingTableIndex, EmberMessageBuffer message, EmberStatus status)
{
  int8u *data = emberMessageBufferContents(message);
  int8u Device = 0;
  //fprintf(stdout,"Got Into Message Sent\r\n");

  switch(data[0])
  {
	case CU_REMOVE:
		if(status == EMBER_SUCCESS){
			//fprintf(stdout,"Is Success\r\n");
			//find in information store by eui64 num and set Remved flag and reset Remove flag
			String_Copy(data, 1, Cipher_Params_CU.Eui64, 0, SIZE_EUI64);
			Device = SearchDeviceArray(data[9]);
			switch(data[9])
			{
				case SENSOR:	SensorSNid[Device].Remove = FALSE;
								SensorSNid[Device].Removed = TRUE;
								SensorRemoved = TRUE;
								//fprintf(stdout,"Got ACK for REMOVE\r\n");
								break;

				case MOBILE:	MobileSNid[Device].Remove = FALSE;
								MobileSNid[Device].Removed = TRUE;
								MobileRemoved = TRUE;
								break;

				case HANDHELD:	HandheldSNid[Device].Remove = FALSE;
								HandheldSNid[Device].Removed = TRUE;
								HandheldRemoved = TRUE;
								break;
			}
		}
		break;
  }


}


// Indicates the success or failure of a connection.
void emberConnectionStatusHandler(int8u bindingTableIndex, int8u status)
{
  fprintf(stdout, "Connection to binding %d: %x\r\n", bindingTableIndex, status);
}

/*
   Indicates the state of the stack. When the stack is not able to communicate
   with the network or is down for some other reason, this function will be called
   by the stack or the gateway-node in the case of a host application. The stack is not
   able to transmit messages unless it reports a status of EMBER_NETWORK_UP.
*/
void emberStackStatusHandler(EmberStatus status)
{
  fprintf(stdout, "agg: rx stackStatus=%d UP=%d DN=%d\n", status, EMBER_NETWORK_UP, EMBER_NETWORK_DOWN);
  if (status == EMBER_NETWORK_UP) {
    networkUp = TRUE;
    Error_Note("networkUp has been set true by emberStackStatusHandler", 0, "N/A");
    //I moved chanel set to part of initialization cuz it doesnt' make sense
    //to do it here, we were just told we were good on whatever the default channel was,
    //then we're going to switch channels? that doesn't make sense.
    //set our channel
    //emberSetRadioChannel(AMR_SAMPLE_CHANNEL);

    //addTimer(SEND_ADVERT_TIMER_INDEX, sendAdvertisement, 0, advertisementPeriod, 1);

  } else if (status == EMBER_NETWORK_DOWN) {
    networkUp = FALSE;
	Error_Note("networkUp has been set false by emberStackStatusHandler", 0, "N/A");
	if(NetworkHasBeenUp) NetworkDownMessage();
    // Delete timer to send aggregator advertisement
    //addTimer(SEND_ADVERT_TIMER_INDEX, NULL, 0, 0, 0);
  }
}


//---------------------------------------------------------------------
/* RNAP SPECIFIC HANDLERS                                            */

// Indicates the rnap status, report the following code
void emberRnapStatusHandler(EmberStatus status)
{
  switch (status) {
  case EMBER_RNAP_MISMATCH:
    fprintf(stdout, "Host and Node binding information mismatched\n");
    break;
  case EMBER_RNAP_ACK_WITH_ERR:
    fprintf(stdout, "Node acked with error code\n");
    break;
  case EMBER_RNAP_COMMUNICATION_FAILED:
    fprintf(stdout, "Rnap communication with the Node failed\n");
    break;
  default:
    fprintf(stdout, "Error Rnap status\n");
    break;

  }
}

// Indicates the failure of sending rnap message.
void emberRnapMessageSent(EmberMessageBuffer rnapMessage, EmberStatus status)
{
  if (status == EMBER_RNAP_DELIVERY_FAILED) {
    fprintf(stdout, "Rnap message failed to be delivered\n");
  }
  else {
		fprintf(stdout, "Error Status for emberRnapMessageSent\n");
		ResetYouGit = TRUE;   //if comms to rf board dies will bug out so nick can restart it, tries 5 times before it gets here also added another
		RnapErrorCount++;
  }
}


//------------------------------------------------------------------------------
// Aggregator Application Functions


//creates and sets a binding to send multicasts to
//or in the case of a meter, receive multicasts from related
//to the amr application
void createMulticastBinding(int8u index)
{
  EmberBindingTableEntry entry;

  entry.type = EMBER_MULTICAST_BINDING;
  entry.identifier[0] = HIGH_BYTE(MULTICAST_GROUP);
  entry.identifier[1] = LOW_BYTE(MULTICAST_GROUP);
  entry.local = ourTerminal;

  assert(emberSetBinding(index, &entry) == EMBER_SUCCESS);
}



void toggleBlinker(int8u timerId, int32u param)
{
  //fprintf(stdout, "*");
}




