/**
   @File: binding-table.c
   @Breif Binding table utility functions definition.
   @see binding-table.h
   @author Guohui Liu, guohui@ember.com

   This Ember Host API library is subject to the Ember corporation
   Development Kit Software License Agreement delivered with this file to
   licensee.
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */
#include <string.h>             // for memcpy
#include "app/host-framework/c/host-api.h"
#include "stack/include/error.h"

// Defined in host-api.c
extern Binding emHostBindingTable[];
extern Connection emHostConnectionTable[]; 

// Finds the binding index for the incoming datagram message.
int8u emFindIncomingDatagramBinding(EmberNodeId remoteNode,
				    EmberTerminal localTerminal,
				    EmberTerminal remoteTerminal)
{
  int8u index;

  for (index = 0; index < EMBER_HOST_BINDING_TABLE_SIZE; index++) {
    Connection *connection = emHostBindingTable[index].remoteNode;
    if (connection != NULL)
      if (connection->state != CON_UNUSED) {
       EmberBindingTableEntry *binding = emHostBindingTable[index].bindingEntry;
        if (binding != NULL && binding->type != EMBER_MULTICAST_BINDING)
	  if (binding->local == localTerminal
	      && binding->remote == remoteTerminal
	      && emHostBindingTable[index].remoteNode->nodeId == remoteNode)
	    return index;
      }
  }
  return NULL_BYTE;
}

// Finds the binding index for the incoming multicast message.
int8u emFindMulticastBinding(int8u start, int16u multicastId)
{
  int8u i;
  EmberBindingTableEntry *entry;

  for (i = start; i < EMBER_HOST_BINDING_TABLE_SIZE; i++) {
    entry  = emHostBindingTable[i].bindingEntry;
    
    if (entry != NULL && entry->type == EMBER_MULTICAST_BINDING) {
      if (entry->identifier[0] == HIGH_BYTE(multicastId)
          && entry->identifier[1] == LOW_BYTE(multicastId))
        return i;
    }
  }
  return EMBER_NULL_BINDING;
}

// Finds the connection index for the given nodeId.
int8u emFindNodeConnectionIndex(EmberNodeId nodeId)
{
  int8u i;

  for (i = 0; i < EMBER_HOST_CONNECTION_TABLE_SIZE; i++) {
    if (emHostConnectionTable[i].nodeId == nodeId 
	&& emHostConnectionTable[i].state != CON_UNUSED) {
      return i;
    }
  }
  return NULL_BYTE;
}

// Finds the binding whose local terminal is the same as the given terminal.
EmberStatus emFindUniqueTerminalBindingIndex(EmberTerminal terminal,
                                             int8u *indexLoc)
{
  int8u i;
  int8u index = NULL_BYTE;
  EmberBindingTableEntry *entry;

  for (i = 0; i < EMBER_HOST_BINDING_TABLE_SIZE; i++) {
    entry = emHostBindingTable[i].bindingEntry;
    if (entry != NULL && entry->type != EMBER_UNUSED_BINDING) {
      if (entry->local == terminal) {
        if (index == NULL_BYTE)
          index = i;
        else
          return EMBER_TERMINAL_HAS_MULTIPLE_BINDINGS;
      }
    }
  }
  if (index == NULL_BYTE)
    return EMBER_INVALID_BINDING_TERMINAL;
  else {
    *indexLoc = index;
    return EMBER_SUCCESS;
  }
}
