/**
   @File rnap-message-queue.h
   @brief Declaration of the message queue operations.
   @see rnap-message-queue.c
   @author Guohui Liu (guohui@ember.com)

   This Ember Host API library is subject to the Ember corporation
   Development Kit Software License Agreement delivered with this file to
   licensee.
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */
#ifndef RNAP_MESSAGE_QUEUE_H
#define RNAP_MESSAGE_QUEUE_H

// Max queue size
#define MAX_RNAP_Q_SIZE 64
// A set of queue operation macro
#define Q_TRIM(index) ((index) & (MAX_RNAP_Q_SIZE - 1))
#define Q_NEXT_INDEX(index) (Q_TRIM(index+1))
#define Q_NEXT(index) (index = Q_NEXT_INDEX(index))
#define Q_ENQUEUE(queue, data)                   \
  queue.rnapMessages[queue.tail] = data;         \
  queue.length++;                                \
  Q_NEXT(queue.tail)
#define Q_DEQUEUE(queue)                         \
  queue.rnapMessages[queue.head];                \
  queue.rnapMessages[queue.head] = NULL;         \
  queue.waitTime = 0;                            \
  queue.length--;                                \
  Q_NEXT(queue.head)

// Data structure for message queue.
typedef struct {
  int8u head;
  int8u tail;
  int8u length;
  boolean needAck;
  int32u waitTime;
  EmberMessageBuffer rnapMessages[MAX_RNAP_Q_SIZE];
} RnapMessageQueue;

// Keeps the last sent rnap message, framing bytes are included.
extern EmberMessageBuffer emLastRnapMessageSent;

// Initializes the message queue.
void emRnapQInit(void);

// Adds one rnap message to the queue. return FALSE if queue is full.
boolean emAddRnapQMessage(EmberMessageBuffer buffer);

// Removes one rnap message from the queue head.
EmberMessageBuffer emRemoveRnapQMessage(void);

// Gets the current queue length.
int8u emGetRnapQSize(void);

// Sets the needAck value
void emSetNeedAck(boolean needAck);

// Gets the needAck value.
boolean emNeedAck(void);

// Increases the wait time and check if max wait reached.
boolean emWaitAckTooLong(int32u timePassed, int32u maxWait);
#endif // RNAP_MESSAGE_QUEUE_H
