/*
This file contains the code that handles the devices, uploading device information into my 
device info structures and search routines to find devices

*/

#define SNUM_COLM		0
#define EUI64_COLM		1
#define RFMODCLSS_COLM	2
#define MODEL_COLM		3

//Will add up to 248 leaving 2 bindings left in table
#define MAX_SENSORS		128
#define MAX_MOBILE		60
#define MAX_HANDHELD	60

//Used in structure below 256 arbitrarly since moster block is only 203 bytes at the mo (01-09-05)
//Monster Block is now set to 255 bytes (4-11-05)
#define MAX_DATASIZE	300

//int CurrentSensor;
//int CurrentMobile;
//int CurrentHandheld;

typedef struct {
        int8u         Model;          //for nick
        int8u         DeviceType;     //SENSOR,MOBILE,HANDHELD
		int8u         bMesher;	      //Whether a constant mesher
		int8u         SerialNum[11];
		int8u         ModelStr[12];
		int8u         FirmwareStr[7]; //Nigels
		int8u         RFFirmwareStr[7];//Mine
		int8u		  Eui64[9];			//only used on CU
	} CUcommission_t;

CUcommission_t Cipher_Params_CU;

typedef struct {
	int8u         	Mesher;								//Weather a constant mesher
	int8u         	SerialNumber[SIZE_SNUM+1];
	int8u         	ModelString[SIZE_MDLSTR+1];
	int8u         	FirmwareString[SIZE_FRMSTR+1];
	int8u 		  	Eui64number[SIZE_EUI64+1];
	int8u			Data[MAX_DATASIZE];
	int8u			Packet1;			//will have to be cleared down after packet built and transfered to nick
	int8u			Packet2;			//will have to be cleared down after packet built and transfered to nick
	int8u			Packet3;			//will have to be cleared down after packet built and transfered to nick
	int8u			Packet4;			//will have to be cleared down after packet built and transfered to nick
	int8u 			RequiredPackets;	//will have to be cleared down after packet built and transfered to nick
	int8u			Remove;				//flag gets set when Nick tells me to delete this transmitter
	int8u			Removed;			//Flag gets set when i have actually removed it
	int8u			BindingTableIndex;
	int8u			Lqi;
	int8u			GotMonsterBlock;
	long			LastPacketRecieved;
} DeviceInformation;

DeviceInformation SensorSNid[MAX_SENSORS];
DeviceInformation MobileSNid[MAX_MOBILE];
DeviceInformation HandheldSNid[MAX_HANDHELD];

boolean SensorRemoved;
boolean MobileRemoved;
boolean HandheldRemoved;

boolean HandlingTransmitter = FALSE;
int8u CurrentTransmitter[9];

int8u CurrentSensor;

int8u DataPacketTimeOut = 15; //15 seconds  

int8u unreg[] = "un-reg";
/*											//SNUM_COLM			EUI64_COLM		RFMODCLSS_COLM			MODEL_COLM		
int8u SensorSNid[MAX_SENSORS][4][16];		// coloum1			coloum2			coloum3					coloum4			
											//	Comark S/n		EUI64Num	Mesher/Non-mesher flag	     Model			


//Currently only support for ?? units each
int8u MobileSNid[MAX_MOBILE][4][16];		// coloum1			coloum2			coloum3					coloum4			
											//	Comark S/n		EUI64Num	Mesher/Non-mesher flag	     Model			

int8u HandheldSNid[MAX_HANDHELD][4][16];	// coloum1			coloum2			coloum3					coloum4			
											//	Comark S/n		EUI64Num	Mesher/Non-mesher flag	     Model			
*/

//is defined in cipher.h wich is called by cipher_defines.h
/*enum {		//devices will be catagorized into the following classes
  //Most transmitters will use this class Eg the standard transmitters supplied with the system
  SENSOR,
  //Mobile devices could be transport stylee
  MOBILE,
  //Auditor RF etc
  HANDHELD,
};			//*/


void longToascii(unsigned long number, int8u *destination);
void String_Copy(int8u *from, int8u from_start_index, int8u *to, int8u to_start_index, int8u bytes);
int8u String_Compare(int8u *string1, int8u string1_start_loc, int8u *string2, int8u string2_start_loc, int8u bytes_to_compare);
int8u ChckDeviceSN(int8u *packet, int8u DevType);
int8u SearchDeviceArray(int8u DevType);
void UpdateDevice(int8u Device, int8u DevType);
int8u SearchDeviceArrayForRemoved(int8u DevType);
int8u find_sensor_binding(int8u *id);
int8u ChckLeafSN(int8u *packet, int8u DevType);

int8u ChckDeviceSN(int8u *packet, int8u DevType)	//loads Device information into the array
{
	int i;

	switch(DevType)
	{
	
	case SENSOR:
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(String_Compare((Cipher_Params_CU.SerialNum), 0, SensorSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					if(String_Compare(SensorSNid[i].Eui64number, 0, unreg, 0, 6))
					{
						//add the EUI64 number
						String_Copy((Cipher_Params_CU.Eui64), 0, SensorSNid[i].Eui64number, 0, SIZE_EUI64); //will copy in byte by byte
						SensorSNid[i].Mesher = Cipher_Params_CU.bMesher;
						String_Copy((Cipher_Params_CU.ModelStr), 0, SensorSNid[i].ModelString, 0, SIZE_MDLSTR);
						CurrentSensor = i;
						SensorSNid[i].LastPacketRecieved = the_time + DataPacketTimeOut;
						return(TRUE); //1 if true 0 if not stop searching
					}
					else
					{
						//we don't have a binding table entry but we do have the comark s/n but EUI64 coloum does not contain "un-reg"
						//which could mean it has a valid eui64 num in it this should not happen!!
						fprintf(stderr, "\n\r\n\rFatal Error!!!!!!\n\r\n\r");
					}
				}
			}
			return(FALSE); //1 if true 0 if not 
			
	case MOBILE:
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(String_Compare((Cipher_Params_CU.SerialNum), 0, MobileSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					if(String_Compare(MobileSNid[i].Eui64number, 0, unreg, 0, 6))
					{
						//add the EUI64 number
						String_Copy((Cipher_Params_CU.Eui64), 0, MobileSNid[i].Eui64number, 0, SIZE_EUI64); //will copy in byte by byte
						MobileSNid[i].Mesher = Cipher_Params_CU.bMesher;
						String_Copy((Cipher_Params_CU.ModelStr), 0, MobileSNid[i].ModelString, 0, SIZE_MDLSTR);
						CurrentSensor = i;
						return(TRUE); //1 if true 0 if not stop searching
					}
					else
					{
						//we don't have a binding table entry but we do have the comark s/n but EUI64 coloum does not contain "un-reg"
						//which could mean it has a valid eui64 num in it this should not happen!!
						fprintf(stderr, "\n\r\n\rFatal Error!!!!!!\n\r\n\r");
					}
				}
			}
			return(FALSE); //1 if true 0 if not 
	
	case HANDHELD:
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(String_Compare((Cipher_Params_CU.SerialNum), 0, HandheldSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					if(String_Compare(HandheldSNid[i].Eui64number, 0, unreg, 0, 6))
					{
						//add the EUI64 number
						String_Copy((Cipher_Params_CU.Eui64), 0, HandheldSNid[i].Eui64number, 0, SIZE_EUI64); //will copy in byte by byte
						HandheldSNid[i].Mesher = Cipher_Params_CU.bMesher;
						String_Copy((Cipher_Params_CU.ModelStr), 0, HandheldSNid[i].ModelString, 0, SIZE_MDLSTR);
						CurrentSensor = i;
						return(TRUE); //1 if true 0 if not stop searching
					}
					else
					{
						//we don't have a binding table entry but we do have the comark s/n but EUI64 coloum does not contain "un-reg"
						//which could mean it has a valid eui64 num in it this should not happen!!
						fprintf(stderr, "\n\r\n\rFatal Error!!!!!!\n\r\n\r");
					}
				}
			}
			return(FALSE); //1 if true 0 if not 
	
	}
	return(FALSE); //1 if true 0 if not 
}

int8u ChckLeafSN(int8u *packet, int8u DevType)	//checks an incomming leaf node advertisment (multicast)
{
	int i;
	int8u Snummm[11];

	String_Copy(packet, 9, Snummm, 0, SIZE_SNUM);
	Snummm[SIZE_SNUM+1] = 0;
	fprintf(stderr, "Serial Number %s\n\r",Snummm);

	switch(DevType)
	{
	
	case SENSOR:
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(String_Compare(packet, 9, SensorSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					return(TRUE); //1 if true 0 if not stop searching
				}
			}
			return(FALSE); //1 if true 0 if not 
			
	case MOBILE:
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(String_Compare(packet, 9, MobileSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					return(TRUE); //1 if true 0 if not stop searching
				}
			}
			return(FALSE); //1 if true 0 if not 
	
	case HANDHELD:
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(String_Compare(packet, 9, HandheldSNid[i].SerialNumber, 0, SIZE_SNUM))  //
				{
					return(TRUE); //1 if true 0 if not stop searching
				}
			}
			return(FALSE); //1 if true 0 if not 
	
	}
	return(FALSE); //1 if true 0 if not 
}

//-------------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------------
//Array Searching Routines,  For a comishioned network Arrays will be searched by eui64 num
//-------------------------------------------------------------------------------------------------------------------


//will find a device based on its eui64 number and return the index location in the array's
int8u SearchDeviceArray(int8u DevType)
{
	int i;

	switch(DevType)
	{
		case SENSOR:
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(String_Compare((Cipher_Params_CU.Eui64), 0, SensorSNid[i].Eui64number, 0, SIZE_EUI64))  //
				{
					//fprintf(stderr, "\n\r\n\rFound Sensor!!!!!!\n\r\n\r");	//debug info
					return(i); //
				}
				else
				{
					//fprintf(stderr, "\n\r\n\rCouldn't Find Sensor!!!!!!\n\r\n\r");
				}
			}
			return(254); //error code, means no match although not really possible to get here
			
		case MOBILE:
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(String_Compare((Cipher_Params_CU.Eui64), 0, MobileSNid[i].Eui64number, 0, SIZE_EUI64))  //
				{
					return(i); //
				}
				else
				{
					//fprintf(stderr, "\n\r\n\rCouldn't Find Mobile!!!!!!\n\r\n\r");
				}
			}
			return(254); //error code, means no match although not really possible to get here
			
		case HANDHELD:
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(String_Compare((Cipher_Params_CU.Eui64), 0, HandheldSNid[i].Eui64number, 0, SIZE_EUI64))  //
				{
					return(i); //
				}
				else
				{
					//fprintf(stderr, "\n\r\n\rCouldn't Find Handheld!!!!!!\n\r\n\r");
				}
			}
			return(254); //error code, means no match although not really possible to get here
	}
	return(254); //error code, means no match although not really possible to get here
}

int8u SearchDeviceArrayForRemoved(int8u DevType)//will return the index location of the device to be removed
{
	int i;

	switch(DevType)
	{
		case SENSOR:
			for(i=0;i<MAX_SENSORS;i++)
			{
				if(SensorSNid[i].Removed)	return(i); //
			}
			return(254); //error code, means no match none to be removed
			
		case MOBILE:
			for(i=0;i<MAX_MOBILE;i++)
			{
				if(MobileSNid[i].Removed)  return(i); //
			}
			return(254); //error code, means no match none to be removed
			
		case HANDHELD:
			for(i=0;i<MAX_HANDHELD;i++)
			{
				if(HandheldSNid[i].Removed)  return(i); //
			}
			return(254); //error code, means no match none to be removed
	}
	return(254); //error code, means no match none to be removed  
}


//this will update information to the snum array
void UpdateDevice(int8u Device, int8u DevType)
{
	switch(DevType)
	{
		case SENSOR:
			SensorSNid[Device].Mesher = Cipher_Params_CU.bMesher;
			String_Copy((Cipher_Params_CU.ModelStr), 0, SensorSNid[Device].ModelString, 0, SIZE_MDLSTR);
			break; //
			
		case MOBILE:
			MobileSNid[Device].Mesher = Cipher_Params_CU.bMesher;
			String_Copy((Cipher_Params_CU.ModelStr), 0, MobileSNid[Device].ModelString, 0, SIZE_MDLSTR);
			break; //
			
		case HANDHELD:
			HandheldSNid[Device].Mesher = Cipher_Params_CU.bMesher;
			String_Copy((Cipher_Params_CU.ModelStr), 0, HandheldSNid[Device].ModelString, 0, SIZE_MDLSTR);
			break; //
	}
}

int8u find_sensor_binding(int8u *id)  //finds the binding information for a recieved EUI64 number and returns the index
{							   		//i could also use this index to locate information in the various cmd arrays		

	EmberBindingTableEntry binding;
    int8u i;
	
	//find its address
	for (i = 0; i < MAX_METERS; i++) // this may be not necassary for this function
	{
		emberGetBinding(i, &binding);
		if (MEMCOMPARE(binding.identifier, (id), EUI_LEN) == 0) 
		{
			return(i);  				   //serial_number_id[current_transmitter][EUI64_coloum]
		}
	}

	return(254);
	
}


//Pauls Version of String copy-(morelike a memcopy)---
//----------------------------------------------------
//Will copy a string from a buffer from any point in that buffer (from_start_index) for x contiguos bytes
//and load them into a buffer starting at location to_start_index
void String_Copy(int8u *from, int8u from_start_index, int8u *to, int8u to_start_index, int8u bytes)
{
  int8u i;
  
  for(i=0;i<bytes;i++)
  {
    to[i+to_start_index] = from[i+from_start_index];
  }
  to[i+to_start_index] = 0;  //null the buffer
  
}

//String Compare----------------------------------
//------------------------------------------------
//compare two strings within two buffers that may contain other infromation.
//need to supply the start location of both strings within the buffers and then the length of the string (bytes_to_compare)
int8u String_Compare(int8u *string1, int8u string1_start_loc, int8u *string2, int8u string2_start_loc, int8u bytes_to_compare)
{
  int8u i, error,result;
  
  i = 0;          //set up start values
  error = FALSE;
  result = TRUE;
  
  do
  {
    if(string1[string1_start_loc+i] != string2[string2_start_loc+i])  //compare byte by byte
    {
      error = TRUE;       //if strings do not match set the false flag, should make it drop out of while loop
      result = FALSE;     //and return the error
    }
    else{i++;}
  
  }
  while((!error) && (i!=bytes_to_compare));   //will drop out on an error or when has compared the required number of bytes
  
  

  return(result);   //will return a 1 if both strings match
}

void longToascii(unsigned long number, int8u *destination)
{
  int8u i = 0;
  int8u small_num;
  
  
  small_num = number/1000000l;				//1Millions
  number = number - (small_num*1000000l);
  if(small_num <10)
  {
	destination[i] = '0';
	i++;
	destination[i] = (small_num + 48);
	i++;
  }
  else
  {
	destination[i] = ((small_num/10) + 48);
	i++;
	destination[i] = ((small_num-((small_num/10)*10)) + 48);
	i++;
  }
  
  small_num = number/100000l;
  number = number - (small_num*100000l);
  destination[i] = (small_num + 48);
  i++;
  
  small_num = number/10000l;
  number = number - (small_num*10000l);
  destination[i] = (small_num + 48);
  i++;
  
  small_num = number/1000l;
  number = number - (small_num*1000l);
  destination[i] = (small_num + 48);
  i++;
  
  small_num = number/100l;
  number = number - (small_num*100l);
  destination[i] = (small_num + 48);
  i++;
  
  small_num = number/10l;
  number = number - (small_num*10l);
  destination[i] = (small_num + 48);
  i++;
  
  small_num = number;
  destination[i] = (small_num + 48);
  i++;
  
  destination[i]=0;  //null it
}




