/**
 * File: cipher-meter-cmds.c
 * Description: meter's commands
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */


//------------------------------------------------------------------------------
// Commands
EmberCommandEntry commands[] = {
  {'f', "find",       FindCu,          "",     ""},
  {'q', "CmdPkt",       CmdPacket,          "b",     ""},
  {'c', "Cmd",       SendCmdtoNigel,          "",     ""},
  {'t', "Trans",       TxDatacmd,          "",     ""},
  {'s', "Status",       R_Status,          "",     ""},
  {'z', "zsleep",       D_Sleep,          "",     ""},
  {'i', "Init",       GetCipherParameters,          "b",     ""},
  {'p', "packet",       packetcmd,          "b",     ""},
  {'b', "bootload",       bootloadCommand,          "",     "bootloader"},
  {'w', "west",       test,          "",     ""},
  { 0, NULL, NULL, NULL, NULL } // NULL action makes this a terminator
};



int16u arguments[4];

EmberCommandState commandState =
  {
    commands,
    commandErrorHandler,
    arguments,
    4,                    // maximum command arguments
    10                    // integer base
  };


//----------------------------------------------------------------
// Standard commands provided by the command interpreter as defined in amr.h


//------------------------------------------------------------------------------
// handle a command to start the bootloader
void bootloadCommand(EmberCommandState *state)
{
  emberSerialGuaranteedPrintf(SERIAL_PORT, "starting bootloader...\r\n");
  microLaunchBootloader(MICRO_LAUNCH_BOOTLOADER_KEY);
}
//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------



//Cipher Application Cmds---------------------------------------------------------
//-------------------------------------------------------------------------------
void R_Status(EmberCommandState *state)
{
  int8u Responce[7];      //response is ["len" "ChkSum" "statByte"]\r  = 6 bytes total +1 for null
  int8u status[1];
    
  status[0] =*(int8u*)&RStatus;
  CheckSumWriteBinaryDelimiter (status, 2, Responce);     //2 because it includes the checksum count
  emberSerialWriteData(SERIAL_PORT, Responce, 6);
}


void GetCipherParameters(EmberCommandState *state)
{
  emberCopyFromLinkedBuffers(state->stringBuffer_loc,0,(globalBuffer+1), state->binaryLength);  //this will copy out all 
                                                                                 //the data without the cmd and I[len]\r
  globalBuffer[0] = state->binaryLength;
  if(!CheckSumVerify (globalBuffer, state->binaryLength)){
      //NAK the cmd checksum does not match
      emberSerialPrintf(SERIAL_PORT,"!\n\r");  
  }
  else{
    MEMCOPY(&Cipher_Params, &globalBuffer[2], sizeof(commission_t));
    String_Copy(firmVer, 0, Cipher_Params.RFFirmwareStr, 0, 6);
    RStatus.bCommissioned = TRUE;
    //ACK the cmd
    emberSerialPrintf(SERIAL_PORT,"A\n\r");  
  }
  
  //Will always route other peoples messages whilst i'm on, it makes things easier later          
  //if((RStatus.bCommissioned) && (!RStatus.bMesher)) emberEnableRouting(FALSE);  //disable routing because your a leaf node
}

void packetcmd(EmberCommandState *state)
{
  int8u PacketIndex;
  int8u DataSize;
  boolean ToBig = FALSE;
  
  emberCopyFromLinkedBuffers(state->stringBuffer_loc,0,(globalBuffer+1), state->binaryLength);  //this will copy out all 
                                                                                 //the data without the cmd and I[len]\r
  globalBuffer[0] = state->binaryLength;
  if((!CheckSumVerify(globalBuffer, state->binaryLength))){
      emberSerialPrintf(SERIAL_PORT,"!C\n\r");  
  }
  
  PacketIndex = globalBuffer[2];  //
  DataSize = globalBuffer[0]-2;   //minus 2 for chksum + pktIdx update the monster block size
  if((PacketIndex+DataSize) > MAXBLOCK_SIZE) ToBig = TRUE;
  if(ToBig){
      //NAK the cmd 
      emberSerialPrintf(SERIAL_PORT,"!B\n\r");  
  }
  else{
      if((DataSize + PacketIndex)>MonsterBlock[0]) MonsterBlock[0] = DataSize + PacketIndex;    
      PacketIndex++;  //i add the 1 because location 0 of monsterblock is (real)data size
      MEMCOPY(&MonsterBlock[PacketIndex], &globalBuffer[3], DataSize);
      //ACK the cmd
      emberSerialPrintf(SERIAL_PORT,"A\n\r");  
  }
}

void CmdPacket(EmberCommandState *state)//Nigels responce to me giving him a cmd
{
  int8u DataSize;
  
  emberCopyFromLinkedBuffers(state->stringBuffer_loc,0,(globalBuffer+1), state->binaryLength);  //this will copy out all 
                                                                                 //the data without the cmd and I[len]\r
  globalBuffer[0] = state->binaryLength;
  if((!CheckSumVerify(globalBuffer, state->binaryLength))){
      emberSerialPrintf(SERIAL_PORT,"!C\n\r");  
  }
  else
  {
    //i am removing the check sum (the len includes the check sum so..)
    DataSize = globalBuffer[0] - 1;      //add the one for the len byte
    MonsterBlock[0] = DataSize;
    MEMCOPY(&MonsterBlock[1], &globalBuffer[2], DataSize);
    TXnow = TRUE;
    emberSerialPrintf(SERIAL_PORT,"A\n\r"); 
  } 
}

void SendCmdtoNigel(EmberCommandState *state)
{
  int8u CmdSize = 0;
  
  globalBuffer[0] = '[';
  CmdSize++;
  MEMCOPY(&globalBuffer[CmdSize], CmdBuff, CmdBuff[0]+1);
  CmdSize = CmdSize + CmdBuff[0] + 1;
  globalBuffer[CmdSize] = ']';
  CmdSize++;
  globalBuffer[CmdSize] = '\r';
  CmdSize++;
  emberSerialWriteData(SERIAL_PORT, globalBuffer, CmdSize);
  RStatus.bCuRecord = FALSE;    
}


void TxDatacmd(EmberCommandState *state)
{
  TXnow = TRUE;
  //ACK the cmd
  emberSerialPrintf(SERIAL_PORT,"A\n\r");  
}

void FindCu(EmberCommandState *state)
{
  FindCU = TRUE;
  //ACK the cmd
  emberSerialPrintf(SERIAL_PORT,"A\n\r");  
}

void D_Sleep(EmberCommandState *state)
{
  Sleep = TRUE;
  emberSerialGuaranteedPrintf(SERIAL_PORT,"A\n\r");
}


//----------------------------------------------------------------
void commandErrorHandler(int8u errorCode)
{
  emberSerialPrintf(SERIAL_PORT,"%p\n\r",emberCommandErrorNames[errorCode]);  
}

void CheckSumWriteBinaryDelimiter (int8u *data, int8u bin_len, int8u *builtdata) //bin_len will include
{                                                                                //a byte for checksum here
  int8u chksum=0;
  int8u i = 1;
  int8u checksum_len = 0;
  
  builtdata[0] = '[';
  builtdata[1] = bin_len; //including checksum
  builtdata[2] = 0;       //this is the checksum location
  MEMCOPY(&(builtdata[3]), data, (bin_len-1));   //bin_len includes the checksum so we want to remove the extra byte
  
  //Checksum includes the bin_len byte so
  checksum_len = bin_len;
  checksum_len++;
  do chksum-= builtdata[i++];
  while (--checksum_len);
  builtdata[2] = chksum;
  builtdata[2+bin_len] = ']';
  builtdata[3+bin_len] = '\r';
  //builtdata[4+bin_len] = 0;   //null the string
}


int8u CheckSumVerify (int8u *data, int8u bin_len) //bin_len will include
{                                                //a byte for checksum here
  int8u chksum=0;
  int8u i = 0;
  int8u checksum_len = 0;
  //Checksum includes the bin_len byte so
  checksum_len = bin_len;
  checksum_len++;
  do chksum+= data[i++];
  while (--checksum_len);
  return(!chksum);        //if all add to zero checksum is good will return 1 (TRUE)
}



//Test commands and utils-----------------------------------------------------------
//----------------------------------------------------------------------------------

void test(EmberCommandState *state)//debug cmd only
{
  //EmberStatus status;

  //RStatus.bMesher = FALSE;
  //RStatus.bConnected = FALSE;
  //RStatus.bCommissioned = TRUE;   //just a bodge for now to test leaf node comish
  //if((RStatus.bCommissioned) && (!RStatus.bMesher)){
     //status = emberEnableRouting(FALSE);  //disable routing because your a leaf node
     //if (status == EMBER_SUCCESS) emberSerialPrintf(SERIAL_PORT,"Not routing\n\r");  
  //}
  
}








