/**
 * File: node-gateway.c
 * Description: a simple node gateway running rnap
 * Author(s): Ezra Hale, ezra@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */


#include PLATFORM_HEADER
#include "stack/include/ember.h"
#include "stack/include/packet-buffer.h"
#include "stack/include/error.h"
#include "stack/include/hal.h"
#include "app/util/serial/serial.h"     // this and the next are for amr.h
#include "app/util/serial/command-interpreter.h"
#include "hal/board/board.h"
#include "hal/micro/micro.h"
#include "node-gateway.h"
#include "app/util/rnap/rnap.h"
#include "app/util/rnap/rnap-app.h"


//connection count and binding table size must be the same
#define EMBER_BINDING_TABLE_SIZE 16
#define EMBER_TRANSPORT_CONNECTION_COUNT        EMBER_BINDING_TABLE_SIZE
#define EMBER_TEMPORARY_BINDING_ENTRIES         EMBER_BINDING_TABLE_SIZE
#define EMBER_MAXIMUM_AGGREGATION_ROUTES        EMBER_BINDING_TABLE_SIZE
#define EMBER_COST_TABLE_SIZE                   1 //make this as small as possible to conserve memory
#define EMBER_PACKET_BUFFER_COUNT               30 // 32 buffers exceeds available memory

#include "stack/include/ember-static-configuration.h"
#include "app/util/rnap/rnap-static-memory.h"

#ifdef SIMULATOR
//some things for the simulator
extern int16u simulatorId;
PGM_NO_CONST PGM_P applicationString = "RNAP Gateway";
#endif

//The stack will complain if we don't do this
int8u emberEndpointCount = 0;
EmberEndpoint emberEndpoints[1];


//a visualization so we can see things are running on the node
static void heartbeat(void) {
  static int16u lastBlinkTime = 0;
  int16u time;
  
  time = microGetSystemTick();
  
  if ( time - lastBlinkTime > 200 /*ms*/ ) {
    lastBlinkTime = time;

    // blink the LEDs
    boardLEDToggle(BOARDLED1);
    boardLEDToggle(BOARDLED2);
  }
}

//extern void (**debugEvent)(void);
//initialize stack and serial input
void initializeEmberStack(void)
{
  EmberStatus status;
  EmberResetType resetInfo = microGetResetInfo();
  boardBoot();
  microBoot();
  microStartSystemTick();
  INTERRUPTS_ON();
  emberSerialInit(1, BAUD_19200, PARITY_NONE, 1);
  emberSerialInit(0, BAUD_19200, PARITY_NONE, 1); //for debug out
  status = emberInit(resetInfo);
  if (status != EMBER_SUCCESS) {
    emberSerialGuaranteedPrintf(SERIAL_PORT, 
      "error returned from emberInit: 0x%x\r\n",
      status);
    assert(FALSE);
  }
  boardLEDToggle(BOARDLED0);
}


//----------------------------------------------------------------
// For running under the simulator we rename main() so that it does not
// conflict with the simulator's main().
#ifdef EMBER_TEST
  int nodeMain(void)
#else
  int main(void)
#endif     
{
  initializeEmberStack();           //init our stack
  emRnapInit();                     //init rnap
  while(TRUE) {

    //feed the watch dog a bone
    __watchdog_reset();
    
    //gotta service the emberTick
    emberTick();

    //beat our heart so they know we're alive
    heartbeat();  
    
    //process rnap input off the serial port
    emProcessRnapInput();

    //since we're using the serial port heavily we have to service the serial buffer
    //cleanup, normally this would be ifdefed for debug_on.
    emberSerialBufferTick();

    //just for internal stuff
    #ifdef EMBER_TEST
      simulatedTimePasses();  
    #endif
  }  
}


//----------------------------------------------------------------
// Standard handlers

EmberStatus emberRemoteSetBindingHandler(int8u index,
                                         EmberBindingTableEntry *entry)
{
  // Don't let anyone mess with our bindings.
  return EMBER_INVALID_BINDING_INDEX;
}

void emberEndpointFoundHandler(EmberInterfaceId *desiredInterface,
                               EmberEUI64 node,
                               EmberNetworkDistance distance)
{
}

void boardButtonCallback(int8u button, int8u state)
{
}


