/*
 * File: em2420.c
 * Description: Hal support functions for the em2420 radio
 *
 * Author(s): Lee Taylor, lee@ember.com
 *            DC Seward, dc@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */

#define ENABLE_BIT_DEFINITIONS
#include <inavr.h>
#include <ioavr.h>

#include PLATFORM_HEADER
#include "stack/include/ember.h"
#include "stack/include/hal.h"
#include "stack/include/hal-em2420.h"

#include "board/board.h"
#include "micro/micro.h"


void emberHalRadioEnableInts(void)
{
  // Rising edge trigger
  //  EICRB |= (BIT(ISC40)|BIT(ISC41));
  // Low level trigger
  EICRB &= ~(BIT(ISC40)|BIT(ISC41));
  emberHalRadioEnableRxInt();

  // Rising edge input capture
  TCCR1B |= (BIT(ICES1));
  // Enable input capture interrupt (SPD)
  TIMSK |= (BIT(TICIE1));
  // Clear the flag by writing one
  TIFR = BIT(ICF1);
}

void emberHalRadioEnableRxInt(void)
{
  // Enable INT4 (FIFOP)
  EIMSK |= (BIT(INT4));
  // Clear the flag by writing one
  EIFR = BIT(INT4); 
}

void emberHalRadioDisableInts(void)
{
  emberHalRadioDisableRxInt();

  // Disable input capture interrupt (SPD)
  TIMSK &= ~(BIT(TICIE1));
  // Clear the flag by writing one
  TIFR = BIT(ICF1);
}

void emberHalRadioDisableRxInt(void)
{
  // Disable INT4 (FIFOP)
  EIMSK &= ~(BIT(INT4));
  // Clear the flag by writing one
  EIFR = BIT(INT4);
}

#ifndef BOOTLOADER

#pragma vector = INT4_vect
__interrupt void radioReceiveInterrupt(void)
{
  emberRadioReceiveIsr();

  if((radFifoPIN & BIT(radFIFOP)) == 0) {
    // the radio ISR is still low.  This means the radio is in a non-
    // recoverable state.  Time to reboot it.
    emberRadioReBoot();
  }
}

#pragma vector = TIMER1_CAPT_vect
__interrupt void radioSpdInterrupt(void)
{
  microSpdIsr();
}

#endif


