// reserved-ram.h
//
// by establishing a common memory "segment" we allow data exchange between applications. 
// for example the evaluation application and the bootloader share id information for 
// over the air bootloading.
//
// we do this by specifying hard coded ram addresses for key data, and a block or "segment"
// of ram which will not be erased by either application when they boot.
//
// there is a potential here for backwards compatilbity issues, since older applications are 
// likely to "reserve" less ram (for example the bootloaders built before change 8406.)
// this is why it's a good idea to reserve slightly more room than needed. although, assuming
// you're only applications are "some app" and the bootloader, and it's only the bootloader 
// that falls behind, and that it never even approaches the ram use limit - maybe not such an issue.
//
// the actual ram reserved block is specified the linker scripts. 
//
#define RESERVED_RAM_BOOTLOADER_KEY     0x10FE    
#define RESERVED_RAM_PC_DIAG            0x10FC
#define RESERVED_RAM_GATEWAY_ID         0x10F4
#define RESERVED_RAM_EUI64_ID           0x10EC
#define RESERVED_RAM_PAN_ID             0x10EA
#define RESERVED_RAM_NODE_ID            0x10E8
#define RESERVED_RAM_CHANNEL            0x10E7
#define RESERVED_RAM_POWER              0x10E6
#define RESERVED_RAM_BANDS              0x10E5
#define RESERVED_RAM_MODE               0x10E4
#define RESERVED_RAM_RESET_CAUSE        0x10E3
#define RESERVED_RAM_RESET_CAUSE_B      0x10E2
#define RESERVED_RAM_RSTACK_H           0x10E1
#define RESERVED_RAM_RSTACK_L           0x10E0
#define RESERVED_RAM_CSTACK_H           0x10DF
#define RESERVED_RAM_CSTACK_L           0x10DE

// there isn't currently an easy way to share these between the linker scripts
// and the c code directly (since they aren't really segments). that said...
// THESE MUST MATCH THE VALUES IN THE LINKER SCRIPTS
#define SRAM_BASE       0x0100   // Start of ram memory
#define SRAM_END        0x10DD   // End of normal ram memory (~4k + SFR offset  - reserved ram)  
#define EXT_SRAM_END    0x7FFF   // Largest possible external sram chip (32k)   


#define RESET_CAUSE_SELF_WDOG      0
#define RESET_CAUSE_ASSERT_BIT     1
#define RESET_CAUSE_CSTACK_BIT     2
#define RESET_CAUSE_RSTACK_BIT     3
#define RESET_CAUSE_BOOTLOADER_BIT 4

#ifndef __IAR_SYSTEMS_ASM__
extern __no_init __root volatile int8u  emResetCause;
extern __no_init __root volatile int8u  emResetCauseMarker;
extern __no_init __root volatile int16u emResetRStack;
extern __no_init __root volatile int16u emResetCStack;
#endif


