/*
 * File: em2420.c
 * Description: Hal support functions for the em2420 radio
 *
 * Author(s): Brooks Barrett, brooks.barrett@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */

#define ENABLE_BIT_DEFINITIONS
#include <inavr.h>
#include <ioavr.h>

#include PLATFORM_HEADER
#include "stack/include/ember.h"
#include "stack/include/hal.h"
#include "stack/include/hal-em2420.h"

#include "board/board.h"
#include "micro/micro.h"


void emberHalRadioEnableInts(void)
{
  // Low level trigger of INT0 (FIFOP)
  MCUCR &= ~(BIT(ISC00)|BIT(ISC01));
  emberHalRadioEnableRxInt();

  // Rising edge input capture
  TCCR1B |= (BIT(ICES1));
  // Enable input capture interrupt (SPD)
  TIMSK |= (BIT(TICIE1));
  // Clear the flag by writing one
  TIFR = BIT(ICF1);
}

void emberHalRadioEnableRxInt(void)
{
  // Enable INT0 (FIFOP)
  GICR |= (BIT(INT0));
  // Clear the flag by writing one
  GIFR = BIT(INT0);
}

void emberHalRadioDisableInts(void)
{
  emberHalRadioDisableRxInt();

  // Disable input capture interrupt (SPD)
  TIMSK &= ~(BIT(TICIE1));
  // Clear the flag by writing one
  TIFR = BIT(ICF1);
}

void emberHalRadioDisableRxInt(void)
{
  // Disable INT0 (FIFOP)
  GICR &= ~(BIT(INT0));
  // Clear the flag by writing one
  GIFR = BIT(INT0);
}

#pragma vector = INT0_vect
__interrupt void radioReceiveInterrupt(void)
{
  emberRadioReceiveIsr();

  if((radPINFIFOP & BIT(radFIFOP)) == 0) {
    // the radio ISR is still low.  This means the radio is in a non-
    // recoverable state.  Time to reboot it.
    emberRadioReBoot();
  }
}

#pragma vector = TIMER1_CAPT_vect
__interrupt void radioSpdInterrupt(void)
{
  microSpdIsr();
}

