/*
 * File: em2420.c
 * Description: Hal support functions for the em2420 radio
 *
 * Author(s): Lee Taylor, lee@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */

#define ENABLE_BIT_DEFINITIONS
#include <inavr.h>
#include <ioavr.h>

#include PLATFORM_HEADER
#include "stack/include/ember.h"
#include "stack/include/hal.h"
#include "stack/include/hal-em2420.h"

#include "board/board.h"
#include "micro/micro.h"


void emberHalRadioEnableInts(void)
{
  // in the 169, the FIFOP is connected to INT0.
  emberHalRadioEnableRxInt();

  // timing sync not supported on the 169 for andover.
  // Rising edge input capture
  //TCCR1B |= (BIT(ICES1));
  // Enable input capture interrupt (SPD)
  // TIMSK |= (BIT(TICIE1));
  // Clear the flag by writing one
  // TIFR = BIT(ICF1);
}

void emberHalRadioEnableRxInt(void)
{
  // Enable INT0 (FIFOP)
  EIMSK |= (BIT(INT0));
  // Clear the flag by writing one
  EIFR = BIT(INT0); 
}

void emberHalRadioDisableInts(void)
{
  emberHalRadioDisableRxInt();

  // timing sync not supported on the 169 for andover.
  // Disable input capture interrupt (SPD)
  //TIMSK &= ~(BIT(TICIE1));
  // Clear the flag by writing one
  //TIFR = BIT(ICF1);
}

void emberHalRadioDisableRxInt(void)
{
  // Disable INT4 (FIFOP)
  EIMSK &= ~(BIT(INT0));
  // Clear the flag by writing one
  EIFR = BIT(INT0);
}

#pragma vector = INT0_vect
__interrupt void radioReceiveInterrupt(void)
{
  emberRadioReceiveIsr();
  if((radPINFIFOP & BIT(radFIFOP)) == 0) {
    // the radio ISR is still low.  This means the radio is in a non-
    // recoverable state.  Time to reboot it.
    emberRadioReBoot();
  }
}

// timing sync not supported for the 169
//#pragma vector = TIMER1_CAPT_vect
//__interrupt void radioSpdInterrupt(void)
//{
//  microSpdIsr();
//}

