// reserved-ram.h
//
// by establishing a common memory "segment" we allow data exchange between applications. 
// for example the evaluation application and the bootloader share id information for 
// over the air bootloading.
//
// we do this by specifying hard coded ram addresses for key data, and a block or "segment"
// of ram which will not be erased by either application when they boot.
//
// there is a potential here for backwards compatilbity issues, since older applications are 
// likely to "reserve" less ram (for example the bootloaders built before change 8406.)
// this is why it's a good idea to reserve slightly more room than needed. although, assuming
// you're only applications are "some app" and the bootloader, and it's only the bootloader 
// that falls behind, and that it never even approaches the ram use limit - maybe not such an issue.
//
// the actual ram reserved block is specified the linker scripts. 
//
#define RESERVED_RAM_PC_DIAG            0x04FE
#define RESERVED_RAM_RESET_CAUSE        0x04FD
#define RESERVED_RAM_RESET_CAUSE_B      0x04FC
#define RESERVED_RAM_RSTACK_H           0x04FB
#define RESERVED_RAM_RSTACK_L           0x04FA
#define RESERVED_RAM_CSTACK_H           0x04F9
#define RESERVED_RAM_CSTACK_L           0x04F8

// there isn't currently an easy way to share these between the linker scripts
// and the c code directly (since they aren't really segments). that said...
// THESE MUST MATCH THE VALUES IN THE LINKER SCRIPTS
#define SRAM_BASE       0x0100   // Start of ram memory.  
//169 internal RAM ends at 0x04FF, define SRAM lower than that for room for reserved
#define SRAM_END        0x04F7   // End of normal ram memory (~1k + SFR offset  - reserved ram)  

#define RESET_CAUSE_SELF_WDOG      0
#define RESET_CAUSE_ASSERT_BIT     1
#define RESET_CAUSE_CSTACK_BIT     2
#define RESET_CAUSE_RSTACK_BIT     3
#define RESET_CAUSE_BOOTLOADER_BIT 4

#ifndef __IAR_SYSTEMS_ASM__
extern __no_init __root volatile int8u  emResetCause;
extern __no_init __root volatile int8u  emResetCauseMarker;
extern __no_init __root volatile int16u emResetRStack;
extern __no_init __root volatile int16u emResetCStack;
#endif


