/*
 * File: em2420.c
 * Description: Hal support functions for the em2420 radio
 *
 * Author(s): Lee Taylor, lee@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */

#define ENABLE_BIT_DEFINITIONS
#include <inavr.h>
#include <ioavr.h>

#include PLATFORM_HEADER
#include "stack/include/ember.h"
#include "stack/include/hal.h"
#include "stack/include/hal-em2420.h"

#include "board/board.h"
#include "micro/micro.h"

//sorry, this looks bad
//the timing below is too precise to handle any other way
//this was all manually tweaked/verified on a scope
//-WBB 9/28/2004
#ifdef AVR8MHZ
#define radPowerOn()                       \
  do{                                      \
    int16u cycles = 0x14d;                 \
    int8u i;                               \
    CLEARBIT(radVregPORT,radVreg);         \
    OUTPUTPINS(radVregDDR,(BIT(radVreg))); \
    while(cycles--) {                      \
      SETBIT(radVregPORT,radVreg);         \
      i=6;                                 \
      do {                                 \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
      } while(--i);                        \
      CLEARBIT(radVregPORT,radVreg);       \
      i=0x14;                              \
      do {                                 \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
        asm("nop");                        \
      } while(--i);                        \
    }                                      \
    SETBIT(radVregPORT,radVreg);           \
  }while(0)
#elif defined(AVR4MHZ)
//Ignore this for now.  It's more of a note of what to do later
//int8u WARNING_Radio_poweron_instead_of_PWM_bringup;
#define radPowerOn()                       \
  do{                                      \
    SETBIT(radVregPORT,radVreg);           \
    OUTPUTPINS(radVregDDR,(BIT(radVreg))); \
  }while(0)
#elif defined(AVR1MHZ)
#define radPowerOn()                       \
  do{                                      \
    int16u cycles = 0x14d;                 \
    CLEARBIT(radVregPORT,radVreg);         \
    OUTPUTPINS(radVregDDR,(BIT(radVreg))); \
    while(cycles--) {                      \
      SETBIT(radVregPORT,radVreg);         \
      asm("nop");                          \
      CLEARBIT(radVregPORT,radVreg);       \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
      asm("nop");                          \
    }                                      \
    SETBIT(radVregPORT,radVreg);           \
  }while(0)
#else
  #error MCU clock not defined!
#endif

#define radPowerOff()                      \
  do {                                     \
    CLEARBIT(radVregPORT,radVreg);         \
    OUTPUTPINS(radVregDDR,(BIT(radVreg))); \
  } while(0)

void emberHalRadioHoldReset(void)
{
  CLEARBIT(radRESETPORT,radnReset);
  OUTPUTPINS(radDDRD, BIT(radnReset));
}

void emberHalRadioReleaseReset(void)
{
  SETBIT(radRESETPORT,radnReset);
}

void emberHalRadioInit(void)
{  
  emberHalRadioHoldReset();
 
  // bring up radio's power regulators
  //0220 boards still need to have the radio power regulator enabled.
  #ifdef BOARD0220
    CLEARBIT(PORTG,2);
    SETBIT(DDRG,2);
  #endif
  radPowerOn();
 
  microDelayMicroSeconds(7500);  // fogbugz: 602
  
  // set PSEL high
  SETBIT(radPORT,radPSEL);
  // set PCLK low
  SETBIT(radPORT,radPCLK);
  SETBIT(radPORT,radPDI);
  // set PCLK, PALE, and PDATA as outputs
  OUTPUTPINS(radDDR, BIT(radPSEL));
  OUTPUTPINS(radDDR, BIT(radPCLK)); 
  OUTPUTPINS(radDDR, BIT(radPDI));
  // Enable SPI, slave, setup(falling)-sample(rising), msb first, sc/4 (1mbit), no interrupt
  SPCR = BIT(SPE) | /*BIT(DORD) | */BIT(MSTR);/* | BIT(CPHA)*/;
  SPSR = BIT(SPI2X); // crank spi to max speed (sclk/2) or 2mbit (assuming 4Mhz sclk)
  
  emberHalRadioReleaseReset();  
}

void emberHalRadioShutdown(void)
{
  CLEARBIT(radRESETPORT,radnReset);
  
  SETBIT(radPORT,radPSEL);
  CLEARBIT(radPORT,radPCLK);
  CLEARBIT(radPORT,radPDI);
  //0220 boards still need to have the radio power regulator disabled.
  #ifdef BOARD0220
    SETBIT(PORTG,2);
  #endif
  radPowerOff();
}

boolean emberHalGetRadioFifoPin(void)
{
  return (radPINFIFO & BIT(radFIFO));
}

boolean emberHalGetRadioFifopPin(void)
{
  return (radPINFIFOP & BIT(radFIFOP));
}

boolean emberHalGetRadioCcaPin(void)
{
  return (radCCAPIN & BIT(radCCA));
}

#ifdef HALTEST
   boolean emberHalGetRadioSpdPin(void)
   {
     return (radSPDPIN & BIT(radSPD));
   }
#endif

// register stuff... move elsewhere?
void radSpiWrite(int8u data)
{
  int8u tempbyte;

  tempbyte = data;  

  SPDR = tempbyte;
  while(!(SPSR & (BIT(7))))
  ;
  SPSR &= ~(BIT(7));// SPIF
     
}

int8u radSpiRead(void)
{
  int8u data = 0;
  int8u tempbyte;
 
  tempbyte = 0x00;  

  SPDR = tempbyte;
  while(!(SPSR & (BIT(7))))
  ;
  SPSR &= ~(BIT(7));// SPIF
     
  data = SPDR;

  return data;
}

int8u radSpiReadWrite(int8u data)
{
   int8u statusdata = 0;
   int8u tempbyte;
     
     tempbyte = data;  

     SPDR = tempbyte;
     while(!(SPSR & (BIT(7))))
     ;
     SPSR &= ~(BIT(7));// SPIF
     
     statusdata = SPDR;
  
  return statusdata;
}

int8u radSetReg(int16u addr, int16u data)
{
    int8u status;
    int8u waddr;
    int8u datalsb,datamsb;

    waddr = addr;
    waddr &= ~(BIT(7));
    waddr &= ~(BIT(6));
    
    datalsb = data;
    datamsb = (data>>8);
    
    ATOMIC(
      CLEARBIT(radPORT,radPSEL);
      status = radSpiReadWrite(waddr);
      radSpiWrite(datamsb);
      radSpiWrite(datalsb);
      SETBIT(radPORT,radPSEL);
    )
    return status;
}
/*
int8u radSetReg(int16u addr, int8u datalsb, int8u datamsb)
{
    int8u status;
    int8u waddr;

    waddr = addr;
    waddr &= ~(BIT(7));
    waddr &= ~(BIT(6));
      
    ATOMIC(
      CLEARBIT(radPORT,radPSEL);
      status = radSpiReadWrite(waddr);
      radSpiWrite(datamsb);
      radSpiWrite(datalsb);
      SETBIT(radPORT,radPSEL);
    )
    return status;
}
*/
int16u radGetReg(int8u addr)
{
  int16u val;
  int16u tempval;
  int8u waddr;

  waddr = addr;
  waddr &= ~(BIT(7));
  waddr |= (BIT(6));
  
  ATOMIC(
    CLEARBIT(radPORT,radPSEL);
    radSpiReadWrite(waddr);
    tempval = radSpiRead();
    tempval = (tempval<<8);
    val = radSpiRead();
    val |= tempval;
    SETBIT(radPORT,radPSEL);
  )
  return val;
}

int8u radStrobe(int8u addr)
{
  int8u status;
  int8u waddr;

  waddr = addr;
  waddr &= ~(BIT(7));
  waddr &= ~(BIT(6));
  

  ATOMIC(
    CLEARBIT(radPORT,radPSEL);
    status = radSpiReadWrite(waddr);
    SETBIT(radPORT,radPSEL);
  )
return status;
}

int8u radWriteFifo(int8u* buff, int8u addr, int8u numbytes)
{
   int8u status;
   int8u ch, i;
   int8u waddr;
   
   waddr = addr;
   waddr &= ~(BIT(7));
   waddr &= ~(BIT(6));
   
   ATOMIC(
     CLEARBIT(radPORT,radPSEL);
     status = radSpiReadWrite((addr));
     for(i=0; i<numbytes; i++) {
       ch = buff[i];
       radSpiWrite(ch);
       // we handle pending ints since this takes about 12us per byte on a 4MHz sysclk
       HANDLE_PENDING_INTERRUPTS();
     }   
     SETBIT(radPORT,radPSEL);
   )
     
   return status;

}

int8u radReadFifo(int8u *buff, int8u addr, int8u numbytes)
{
  int8u status;
  int8u i;
  int8u waddr;
  
  waddr = addr;
  waddr &= ~(BIT(7));
  waddr |= (BIT(6));

  ATOMIC(
    CLEARBIT(radPORT,radPSEL);
    status = radSpiReadWrite(waddr);
    for(i=0; i<numbytes; i++) {
      buff[i] = radSpiRead();
      // we handle pending ints since this takes about 12us per byte on a 4MHz sysclk
      HANDLE_PENDING_INTERRUPTS();
    }   
    SETBIT(radPORT,radPSEL);
  )
  return status;
}

int8u radStartReadFifo(int8u addr)
{
  int8u status;
  int8u waddr;
  
  waddr = addr;
  waddr &= ~(BIT(7));
  waddr |= (BIT(6));
  
  CLEARBIT(radPORT,radPSEL);
  status = radSpiReadWrite(waddr);
    
  return status;
}

void radEndReadFIFO(void) 
{
   SETBIT(radPORT,radPSEL);
}

int8u radStartWriteRAM(int16u addr) 
{
      int8u status;
      int8u addresslsb;
      int16u tempaddr;
      int8u addressmsb;
      
      addresslsb = addr;
      addresslsb |= BIT(7);
      tempaddr = (addr >> 1);
      addressmsb = tempaddr;
      addressmsb &= ~(BIT(5));
      
      CLEARBIT(radPORT,radPSEL);
      status=radSpiReadWrite(addresslsb);
      radSpiWrite(addressmsb);
      
      return status;
}

int8u radStartReadRAM(int16u addr) 
{
      int8u status;
      int8u addresslsb;
      int16u tempaddr;
      int8u addressmsb;
      
      addresslsb = addr;
      addresslsb |= BIT(7);
      tempaddr = (addr >> 1);
      addressmsb = tempaddr;
      addressmsb |= (BIT(5));
           
      CLEARBIT(radPORT,radPSEL);
      status=radSpiReadWrite(addresslsb);
      radSpiWrite(addressmsb);
      
      return status;
}

void radEndRAM(void) 
{
   SETBIT(radPORT,radPSEL);
}

int8u radWriteRAM(int8u* buff, int16u addr, int8u numbytes)
{
   int8u status;
   int8u ch, i;
   int8u addresslsb;
   int16u tempaddr;
   int8u addressmsb;
   
   addresslsb = addr;
   addresslsb |= BIT(7);
   tempaddr = (addr >> 1);
   addressmsb = tempaddr;
   addressmsb &= ~(BIT(5));
   
   ATOMIC(
     CLEARBIT(radPORT,radPSEL);
     status=radSpiReadWrite(addresslsb);
     radSpiWrite(addressmsb);
     
     for(i=0; i<numbytes; i++) {
       ch = buff[i];
       radSpiWrite(ch);
       HANDLE_PENDING_INTERRUPTS();
     }   
     
     SETBIT(radPORT,radPSEL);
   )
     
   return status;

}

int8u radReadRAM(int8u* buff, int16u addr, int8u numbytes)
{
  int8u status;
  int8u i;
  int8u addresslsb;
  int16u tempaddr;
  int8u addressmsb;
  
  addresslsb = addr;
  addresslsb |= BIT(7);
  tempaddr = (addr >> 1);
  addressmsb = tempaddr;
  addressmsb |= (BIT(5));

  ATOMIC(
    CLEARBIT(radPORT,radPSEL);
    status=radSpiReadWrite(addresslsb);
    radSpiWrite(addressmsb);
      
    for(i=0; i<numbytes; i++) {
      buff[i] = radSpiRead();
      HANDLE_PENDING_INTERRUPTS();
    }   
      
    SETBIT(radPORT,radPSEL);
  )
    
  return status;
}
