/*
 * File: token-def.c
 * Description:  Defines the tokens as EEPROM variables when running on
 *               hardware.
 *
 * Author(s): Ed Hill, ed@ember.com
 *
 * Copyright 2004 by Ember Corporation. All rights reserved.                *80*
 */

#include PLATFORM_HEADER
#include "stack/include/ember.h"

#ifndef SIMULATOR
// When running on hardware, the tokens are defined here.
// When running in the simulator, the tokens are defined in token.c.

// This cannot be done in token.c because the declaration here is incompatible
// with the declaration in token.h. The difference is the "const". It is
// required here so that the EEPROM location and initial values can be
// specified. The "const" could be replaced by "__no_init" (allowing the
// definition to be done in token.c) but then the EEPROM initialization file
// would have to be generated manually. See the documentation on "#pragma
// location" for more details.
#define TOKEN_MFG TOKEN_DEF
#define TOKEN_DEF(name, type, size, loc, ...)    \
  NO_STRIPPING const EEPROM type TOKEN_ ## name size @ loc = __VA_ARGS__;
#include "stack/config/token-stack.h"
#undef TOKEN_DEF
#endif
