/** @file ember-static-configuration-leaf.h
 *
 * @brief User-configurable stack memory allocations, convenience stubs 
 * for little-used callbacks
 *
 * \b Note: Application developers should \b not modify any portion
 * of this file. Doing so may lead to mysterious bugs. Allocations should be 
 * adjusted only with the macros, as described below. 
 *
 * The main source file of an application should include this file in
 * order to allocate memory required by the stack. <b>It should only be 
 * included a single time for any given application.</b>
 *
 * All allocations have default sizes, so simply including the file is
 * enough for many applications. You can, however, override these defaults
 * by #defining the appropriate macro to a different size.
 *
 * The convenience stubs provided in this file can be overridden by defining
 * the appropriate macro and providing the corresponding callback function.
 * For example, an application employing beacons needs to implement 
 * emberIncomingBeaconHandler() to process incoming beacons. Along with
 * the function definition, the application should provide the following
 * line before the inclusion of this file:
 *
 * #define EMBER_APPLICATION_HAS_INCOMING_BEACON_HANDLER
 *
 * <!-- Author(s): Lee Taylor, lee@ember.com -->
 *
 * <!--Copyright 2004 by Ember Corporation. All rights reserved.            *80*-->
 */

#ifndef DOXYGEN_SHOULD_SKIP_THIS
// ------------- Convenience Stubs -----------

#ifndef EMBER_APPLICATION_HAS_INCOMING_BEACON_HANDLER
// Note that there are two different versions of the beacon handler because
// the leaf stack doesn't provide buffer support.  
void emberIncomingBeaconHandler(EmberEUI64 sender, int8u* payload) {}
#endif // USER_INCOMING_BEACON_HANDLER


#endif //DOXYGEN_SHOULD_SKIP_THIS

