#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""This Program attempts to duplicate the functionality of the original
Kantaris c code, rather than a wholesale rewrite.
Hopefully this code will become an excellent base upon which functionality mods
would become trivial
"""

from ctypes import Structure, c_char, c_bool, c_uint8, sizeof

from cipherexception import MarvinApiError
from cipherconstants import STR_FILE_CORRUPT
from cipherdirectories import ALARMGROUPLIST_FILE


class GroupNameStr(Structure):
    _fields_ = [ ("Name", c_char * 33) ]


class EmailNoStruct(Structure):
    _fields_ = [ ("emailNo", c_uint8 * 20) ]


class AlarmGroupStruct (Structure):

    # typedef  struct {
    #     char        Name[64][33];
    #     char        emailNo[64][20];
    #     char        emailOnAlrm[64];
    #     char        emailOnClr[64];
    #     char        sw1[64];
    #     char        sw2[64];
    #     char        led[64];
    #     char        spk[64];
    #     char        loBat[64];
    #     char        Fault[64];
    #     char        lostMains[64];
    #     char        noSignal[64];
    #     char        row[64];
    #     char        col[64];
    #     char        spare4[64];
    # }   locationsList_t;

    _pack_ = 1
    _fields_ = [
                ("Name", GroupNameStr * 64),
                ("emailNo", EmailNoStruct * 64),
                ("emailOnAlrm", c_bool * 64),
                ("emailOnClr", c_bool * 64),
                ("sw1", c_bool * 64),
                ("sw2", c_bool * 64),
                ("led", c_bool * 64),
                ("spk", c_bool * 64),
                ("loBat", c_bool * 64),
                ("Fault", c_bool * 64),
                ("lostMains", c_bool * 64),
                ("noSignal", c_bool * 64),
                ("skip", c_bool * 3 * 64),
                ]


def load_alarmgrouplist():
    """Load Alarm Group List from Disk - Return a C Structure"""

    grouplist = AlarmGroupStruct()
    try:
        gwf = open (ALARMGROUPLIST_FILE, 'rb')
        number_bytes_read = gwf.readinto(grouplist)
        if number_bytes_read != sizeof(AlarmGroupStruct):
            raise MarvinApiError(STR_FILE_CORRUPT + ALARMGROUPLIST_FILE)
        gwf.close()
    except IOError:
        raise MarvinApiError("File not found: " + ALARMGROUPLIST_FILE)
    return grouplist


def test():
    """Test Me"""

    grouplist = load_alarmgrouplist()
    print grouplist.Name[0].Name
    print grouplist.Name[1].Name
    print grouplist.Name[2].Name
    for i in range(20):
        print grouplist.emailNo[0].emailNo[i]

    # print grouplist.emailNo[0].emailNo[19]


if __name__ == '__main__':
    test()
