#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
Marvin API helper functions

@author: "Nigel Winterbottom"
"""

from __future__ import print_function
import json


class Service_ReturnCodes:
    """Result Messages Enumeration"""
    (ASR_ALL_OK,
     ASR_WARNING_ACCOUNT_DUE_TO_EXPIRE,
     ASR_INVALID_USERNAME,
     ASR_INVALID_PASSWORD,
     ASR_ACCOUNT_EXPIRED,
     ASR_ACCOUNT_DISABLED,
     ASR_UNKNOWN_ERROR,
     ASR_AUDIT_FAILURE,
     ASR_UNKNOWNTYPE,
     ASR_EMAIL_CREATE_ERROR
    ) = range(0, 10)


def create_return_template():
    return {'Data' : "",
            'SysError' : False,
            'AppError' : Service_ReturnCodes.ASR_ALL_OK,
            'Warning' : ""
            }

def build_system_error(message):
    marvin_dto = create_return_template()
    marvin_dto['SysError'] = True
    marvin_dto['Warning'] = message
    return serve_marvin_dto(marvin_dto)

def serve_marvin_dto(marvin_dto):
    return '(%s)' % json.dumps(marvin_dto, skipkeys=False, sort_keys=True, ensure_ascii=True, check_circular=True, allow_nan=True, cls=None, indent=None, separators=None, encoding="utf-8", default=None)

def set_http_ok_response(response, lastmod=None):
    response.set_header('Content-type', 'application/json')
    response.set_header('Cache-Control', 'must-revalidate')
    response.add_header('Cache-Control', 'no-cache, no-store')
#     response_headers = [
#                         ('Content-type', 'text/plain; charset=UTF8'),
#                         # ('Content-Length', str(len(response_body))),
#                         # ('ETag', 'abcdef'),
#                         # ('Last-Modified', lastmod.strftime('%a, %d %b %Y %H:%M:%S GMT')),
#                         ('Cache-Control', 'must-revalidate'),
#                         # ('Pragma', 'no-cache')
#                        ]
    if lastmod:
        response.set_header('Last-Modified', lastmod.strftime('%a, %d %b %Y %H:%M:%S GMT'))



def build_http_not_modified_response():
    status = '304 Not Modified'
    response_headers = []
    return (status, response_headers)

