#!/usr/bin/python
# -*- coding: UTF-8 -*-

'''
This module defines the various types used throughout Nigel's Transmitter

@author: "Nigel Winterbottom"
'''



class TxModel:
    """The Transmitter Models Enumeration"""
    (MDL_PST,
    MDL_PRT,
    MDL_RH,
    MDL_ANA,
    MDL_NWPST,
    ) = range(0, 5)

    @classmethod
    def to_alt_str(cls, ordinal):
        return ('RF512', 'RF516', 'RF513', 'RF515', 'RF542')[ordinal]


class ChanType:
    """Transmitter Channel Enumerated Type"""
    (INT,
     EXT1,
     EXT2,
     DOOR,
     EXT3,
     EXT4,
    ) = range(0, 6)


class ScaleRH:
    """Humidity Channel Scale Code Enumeration"""
    (SCL_TEST,
     SCL_RH,
     SCL_DP,
     SCL_DPC,
     SCL_DPF,
     SCL_DPK,
    ) = range(0, 6)

    @classmethod
    def to_alt_str(cls, ordinal):
        return ('?', '%rh', 'dp', 'dp°C', 'dp°F', 'dp K')[ordinal]


class ScaleT:
    """Temperature Scale Code Enumeration"""
    (SCL_TEST,
     SCL_C,
     SCL_F,
     SCL_K,
    ) = range(0, 4)

    @classmethod
    def to_alt_str(cls, ordinal):
        return ('?', '°C', '°F', 'K')[ordinal]


class RdgCode():
    """Nigel's Transmitter Reading Code Enumeration"""

#     RDG_SKIP("SKIP"),               //Reading to be Trashed ... Sent for Packet Padding if Channel OFF     Babelfish
#     RDG_OK(""),                     //Normal OK Reading ... Show in Table & Graph                        Babelfish
#     RDG_HI("HI ALARM"),             //Reading in high limit                                                Babelfish
#     RDG_LO("LOW ALARM"),            //Reading in low  limit                                                Babelfish
#     RDG_HITRIG("HI ALARM"),         //Channel going into high alarm (URGENT: Send email, sound Klaxon)    Babelfish
#     RDG_LOTRIG("LOW ALARM"),        //Channel going into low  alarm (URGENT: Send email, sound Klaxon)    Babelfish
#     RDG_HIALM("HI ALARM"),          //Reading in high limit after alarm raised                            Babelfish
#     RDG_LOALM("LOW ALARM"),         //Reading in low  limit after alarm raised                            Babelfish
#     RDG_OVR("OVER RANGE"),          //Channel OverRange etc.                                            Babelfish
#     RDG_UNR("UNDER RANGE"),         //Channel UnderRange etc.                                            Babelfish
#     RDG_FAULT("FAULT");               //Sensor Fault on this Channel. (URGENT: Send email, sound Klaxon)    Babelfish

    (RDG_SKIP,
     RDG_OK,
     RDG_HI,
     RDG_LO,
     RDG_HITRIG,
     RDG_LOTRIG,
     RDG_HIALM,
     RDG_LOALM,
     RDG_OVR,
     RDG_UNR,
     RDG_FAULT) = range(0, 11)

    @classmethod
    def to_alt_str(cls, ordinal):
        return ('SKIP', '', 'HI_LIMIT', 'LOW LIMIT', 'HI ALARM TRIG', 'LOW ALARM TRIG',
                'HI ALARM', 'LOW ALARM', 'OVER RANGE', 'UNDER RANGE', 'FAULT')[ordinal]


class RdgStatus:
    rdgcode = RdgCode.RDG_SKIP
    bPeakRdg = False

    def __init__(self, from_integer=0):
        if (from_integer & 0x80):
            self.bPeakRdg = True
            from_integer &= 0x7F
        self.rdgcode = from_integer

