#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""This Program attempts to duplicate the functionality of the original
Kantaris c code, rather than a wholesale rewrite.
Hopefully this code will become an excellent base upon which functionality mods
would become trivial
"""

# from __future__ import print_function
from ctypes import Structure, c_char, sizeof

from cipherexception import MarvinFileError
from cipherconstants import STR_FILE_CORRUPT
from cipherdirectories import VERSION_FILE, COLLECTION_UNIT_ID_FILE

class GatewayIDStruct (Structure):
    _pack_ = 1
    _fields_ = [
                ("sno", c_char * 10),
                ("name", c_char * 32),
                ("version", c_char * 32)
               ]


def rd_versions():
    """Load Version String from File
    Return a single line string
    """
    try:
        gwf = open (VERSION_FILE, 'r')
        vn_id = gwf.readline()
        gwf.close()
    except IOError:
        vn_id = "Error Reading Version"
    return vn_id


def rd_cu_id():
    """Load Gateway ID from Disk - Return a C Structure
    @raise MarvinFileError: If the ID file cannot be read.
    """

    gw_id = GatewayIDStruct()
    try:
        gwf = open (COLLECTION_UNIT_ID_FILE, 'rb')
        number_bytes_read = gwf.readinto(gw_id)
        if number_bytes_read != sizeof(GatewayIDStruct):
            raise MarvinFileError(STR_FILE_CORRUPT + COLLECTION_UNIT_ID_FILE)
        gwf.close()
    except IOError:
        raise MarvinFileError("File not found: " + COLLECTION_UNIT_ID_FILE)
    return gw_id


def test():
    """Test Me"""

    gw_id = rd_cu_id()
    print gw_id.sno
    print gw_id.name
    print gw_id.version
    print rd_versions()


if __name__ == '__main__':
    test()
