"""
This Program presents a login page similar to the Marvin login dialogue.

@author: Nigel Winterbottom
@organization: Comark Instruments
@since: 2014-04-07
"""

from bottle import request, template, abort, redirect
import gatewayId
from cipherexception import MarvinFileError, MarvinApiError
import useraccount

# app = Bottle()

# Expecting a query string like ?u=4
# @app.route('/', method='GET')
def login_form():

    # Read Request Variables
    origin = request.query.get('origin')
    expiry = request.query.get('xp', '0')
    pwfail = request.query.get('pw', '0')

    # Read Stuff off disk
    try:
        gwid = gatewayId.rd_cu_id()
        gateway_vn = gatewayId.rd_versions()
    except MarvinFileError as e:
        abort(404, e)

    message = ""
    if expiry != '0':
        message = "Your RF500 Session has expired. You must Sign In again"
    if pwfail != '0':
        message = 'Username & Password do not Match. Retype password and Retry'

    return template('login',
                    {
                     'gateway_name': gwid.name,
                     'serial': gwid.sno,
                     'version': gateway_vn,
                     'origin': origin,
                     'error_message': message,
                    }
                    )

# @app.route('/', method='POST')
def login_action():

    # Read Request Variables
    password = request.forms.get('pw')
    username = request.forms.get('uname')
    origin = request.forms.get('origin')

    user = useraccount.UserAccount()
    try:
        user.loaduseraccount(name=username)
    except MarvinApiError:
        redirect('../login?origin=%s&pw=1' % origin, 302)

    if user.signin_user(password):
        # Ensure the userID is passed to the redirect
        if 'u=' not in origin:
            param = 'u=%d' % user.account.userID
            if '?' not in origin:
                origin = origin + '?' + param
            else:
                origin = origin + '&' + param
        redirect(origin, 302)
    else:
        redirect('../login?origin=%s&pw=1' % origin, 302)


def redirect_to_login_form(origin, expiry=False):
    """issue a redirect to this page."""
    exp = ''
    if expiry:
        exp = '&exp=1'
    redirect('../login?origin=%s%s' % (origin, exp), 302)
