#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""This Program attempts to duplicate the functionality of the original
Kantaris c code, rather than a wholesale rewrite.
Hopefully this code will become an excellent base upon which functionality mods
would become trivial
"""

from ctypes import Structure, c_char, c_bool, c_uint8, c_uint32, sizeof

from cipherexception import MarvinApiError
from cipherconstants import STR_FILE_CORRUPT
from cipherdirectories import PERIOD_SUMMARY_FILE_PATTERN


class PeriodSummaryStruct (Structure):

    # typedef  struct {
    #     long    startTime;
    #     long    stopTime;
    #     char    TxVersion[33];
    #     long    stopCmd;
    #     long    unAckAlarm;
    #
    #     char    signDone;
    #     char    user[40];
    #     char    reason[51];
    #     char    authority[51];
    #     long    signDate;
    #     char    spare[256-33-4-4-1-40-51-51-4];
    # } periodStartStop_t;

    _pack_ = 1
    _fields_ = [
                ("startTime", c_uint32),
                ("stopTime", c_uint32),
                ("TxVersion", c_char * 33),
                ("stopCmd", c_uint32),
                ("unAckAlarm", c_uint32),
                ("signDone", c_uint8),
                ("user", c_char * 40),
                ("reason", c_char * 51),
                ("authority", c_char * 51),
                ("signDate", c_uint32),
                ("skip", c_bool * 68),
               ]


def load_period_summary(sno, task, period):
    """Load Period Summary from Disk - Return a C Structure"""

    period_summary = PeriodSummaryStruct()
    filepath = PERIOD_SUMMARY_FILE_PATTERN % (sno, task, period)
    try:
        gwf = open (filepath, 'rb')
        number_bytes_read = gwf.readinto(period_summary)
        if number_bytes_read != sizeof(PeriodSummaryStruct):
            raise MarvinApiError(STR_FILE_CORRUPT + filepath)
        gwf.close()
    except IOError:
        raise MarvinApiError("File not found: " + filepath)
    return period_summary


def test():
    """Test Me"""

    period_summary = load_period_summary('CA03110485', 5, 1)
    print period_summary.TxVersion
    print period_summary.signDone
    print period_summary.user
    print period_summary.reason
    print period_summary.authority


if __name__ == '__main__':
    test()
