#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""Implement Read Gateway file"""

from __future__ import print_function
from cgi import parse_qs
import base64
import json
import datetime
import os.path
import math
import api_marvin


class Service_ReturnCodes:
    """Result Messages Enumeration"""
    (ASR_ALL_OK,
     ASR_WARNING_ACCOUNT_DUE_TO_EXPIRE,
     ASR_INVALID_USERNAME,
     ASR_INVALID_PASSWORD,
     ASR_ACCOUNT_EXPIRED,
     ASR_ACCOUNT_DISABLED,
     ASR_UNKNOWN_ERROR,
     ASR_AUDIT_FAILURE,
     ASR_UNKNOWNTYPE,
     ASR_EMAIL_CREATE_ERROR
    ) = range(0, 10)


def build_http_ok_response(lastmod=None):
    status = '200 OK'
    response_headers = [
                        ('Content-type', 'text/plain; charset=UTF8'),
                        # ('Content-Length', str(len(response_body))),
                        # ('ETag', 'abcdef'),
                        # ('Last-Modified', lastmod.strftime('%a, %d %b %Y %H:%M:%S GMT')),
                        ('Cache-Control', 'must-revalidate'),
                        # ('Pragma', 'no-cache')
                       ]
    if lastmod:
        response_headers.append( ('Last-Modified', lastmod.strftime('%a, %d %b %Y %H:%M:%S GMT')) )
    return (status, response_headers)


def build_http_not_modified_response():
    status = '304 Not Modified'
    response_headers = []
    return (status, response_headers)


def application(environ, start_response):

    returner = api_marvin.create_return_template()

    try:
        query = parse_qs(environ.get('QUERY_STRING'))
        filename = query.get('rdfile')[0]

        mt = os.path.getmtime(filename)
        mod_time = datetime.datetime.fromtimestamp(math.floor(mt))

        dtf = '%a, %d %b %Y %H:%M:%S %Z'
        mod_since_rq_str = environ.get('HTTP_IF_MODIFIED_SINCE')  # 'Wed, 06 Nov 2013 21:44:04 GMT'
        if mod_since_rq_str is None:
            mod_since_rq = datetime.datetime.min
        else:
            mod_since_rq = datetime.datetime.strptime(mod_since_rq_str, dtf)

        if mod_time > mod_since_rq:
            returner['Data'] = dumpfile(filename)
            (status, response_headers) = build_http_ok_response(mod_time)
        else:
            (status, response_headers) = build_http_not_modified_response()


    # except IOError as exc:
    #     returner['SysError'] = True
    #     returner['Warning'] = exc.args[1]
    except Exception as exc:
        returner['SysError'] = True
        returner['Warning'] = str(exc)
        (status, response_headers) = build_http_ok_response()

    # print 'Content-type: text/json; charset=UTF8\n'
    # print 'Content-type: text/javascript; charset=UTF8\n'
    # print('Content-Type: text/javascript')

    if ('200' in status):
        response_body = '(%s)' % json.dumps(returner, skipkeys=False, ensure_ascii=True, check_circular=True, allow_nan=True, cls=None, indent=1, separators=None, encoding="utf-8", default=None)
    else:
        response_body = ''

    start_response(status, response_headers)
    return [response_body]


# def serve_system_error(start_response, msg='System Error'):
#     returner = create_return_template()
#     returner['SysError'] = True
#     returner['Warning'] = msg
#     response_body = '(%s)' % json.dumps(returner, skipkeys=False, ensure_ascii=True, check_circular=True, allow_nan=True, cls=None, indent=1, separators=None, encoding="utf-8", default=None)
#     status = '200 OK'
#     response_headers = [
#                         ('Content-type', 'text/plain; charset=UTF8'),
#                         # ('Content-Length', str(len(response_body))),
#                         # ('Cache-Control', 'no-storee'),
#                         # ('Pragma', 'no-cache')
#                        ]
#     start_response(status, response_headers)
#     return [response_body]



def dumpfile(filename=''):
    """Open and convert file to Base-16"""
    source = open(filename, 'rb')
    raw = source.read()
    # return base64.standard_b64encode(raw))
    return base64.b16encode(raw)


if __name__ == '__main__':
    """test drive"""
    environ = {'QUERY_STRING' : 'rrdfile=readfilebase16.pycc'}
    # environ = {'QUERY_STRING' : ''}
    print (application(environ, print))
