"""
Marvin Reporter helper functions

@author: "Nigel Winterbottom"
@since: 2014-04-07
"""

import time
import urllib

from bottle import abort, request
from cipherexception import MarvinApiError, MarvinFileError
import login
import useraccount

def user_signed_in_check(param_u):
    """Parse the u parameter to determine the userID
    and check that he/she is currently signed-in.
    Abort on errors. Redirect to login form if not signed-in.
    """
    user = useraccount.UserAccount()
    try:
        userid = int(param_u)
        user.loaduseraccount(uid=userid)
    except ValueError:
        abort(400, 'Non Numeric UserId')
    except (MarvinApiError, MarvinFileError) as e:
        abort(400, e)

    # Is user Logged In ?
    if user.account.lastIn == 0:
        login.redirect_to_login_form(urllib.quote_plus(request.url))

    if (time.time() - user.account.lastIn) > useraccount.SESSION_INACTIVITY_TIMEOUT:
        login.redirect_to_login_form(urllib.quote_plus(request.url), expiry=True)

    return user
