#!/usr/bin/python
# -*- coding: UTF-8 -*-

'''
Produce an HTML Report with details of every User registered on an RF500A

@since: 2014-01-05
@author: "Nigel Winterbottom"
'''
from bottle import Bottle, request, template, abort
from cipherexception import MarvinFileError
from report_helper import user_signed_in_check
import datetime
import gatewayId
import useraccount

app = Bottle()

# Expecting a query string like ?u=4
@app.route('/')
def users_report():

    # Read Request variables
    param_u = request.query.get('u', '0')
    user = user_signed_in_check(param_u)

    # Read Stuff off disk
    try:
        gwid = gatewayId.rd_cu_id()
        gateway_vn = gatewayId.rd_versions()
    except MarvinFileError as e:
        abort(404, e)
        # return template('fileerror', message=e)

    # Beware Nick's first 3 users are 'none', 'comark', 'System'. These must NOT be shown in the table
    users = [UserProperties(usr.account) for usr in useraccount.iter_users() if usr.account.userID >= 3]

    return template('report_users',
                    show_password=(user.account.userName == 'comark'),
                    gateway_name=gwid.name,
                    gateway_sno=gwid.sno,
                    gateway_vn=gateway_vn,
                    users=users,
                    )


class UserProperties:
    """The Modelled essentials of User Details without User-Preferences needed for my Views"""

    def __init__(self, account=useraccount.UserAccountStruct()):
        """Pass in a useraccount.UserAccountStruct() instance"""
        self.user_id = account.userID
        self.username = account.userName
        self.password = account.userPassword
        self.expiredate = datetime.datetime.utcfromtimestamp(account.expireDate)
        self.restricted_alarmgroups = [bool(grp_byte) for grp_byte in account.restrictLocation]
        self.bActive = not bool(account.removed)
        self.bAdministrator = bool(account.admin)
        self.bAllow_set_task = bool(account.allow_setTask)
        self.bAllow_ack_alarm = bool(account.allow_ackAlarms)
        self.bAllow_add_tx = bool(account.allow_addTx)
        self.bAllow_allow_start_stop = bool(account.allow_startStop)
        self.bAllow_sign_data = bool(account.allow_sign)


if __name__ == '__main__':
    pass
