#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""This Program attempts to duplicate the functionality of the original
Kantaris c code, rather than a wholesale rewrite.
Hopefully this code will become an excellent base upon which functionality mods
would become trivial
"""

from ctypes import Structure, c_char, sizeof, c_bool, c_float, \
    c_int8, c_uint8, c_int16, c_int32, c_uint32

from cipherexception import MarvinApiError
from cipherdirectories import TX_TASK_LIST_FILE_PATTERN


class LimitStruct(Structure):

#     typedef struct {long Hi; long Lo;} limit_t ;                //Alarm limits

    _fields_ = [
                ("Hi", c_int32),
                ("Lo", c_int32),
               ]



class ZoneLimitsStruct (Structure):

#     typedef  struct {
#         limit_t        limit[8];
#     }     Zones_Limits_t;

    _fields_ = [("limit", LimitStruct * 8)]


class RateSetStruct(Structure):

#     //Historic Structure but still half used because Nick refuses to change his code.
#     typedef struct  {
#         unsigned char LogRate;          //In Minutes
#         unsigned char TaskNo_Lo;        //Lo byte of TaskNo
#         unsigned char DoorAveInterval;  //In Minutes
#         unsigned char TaskNo_Hi;        //Hi byte of TaskNo
#         } rate_set_t;

    _fields_ = [
                ("LogRate", c_uint8),
                ("TaskNo_Lo", c_uint8),
                ("DoorAveInterval", c_uint8),
                ("TaskNo_Hi", c_uint8),
               ]



class ChannelFlagStruct(Structure):
    _pack_ = 1
    _fields_ = [
                ("bChan_En", c_uint8, 1),
                ("bAlarm_En", c_uint8, 1),
                ("bEmailOnAlm", c_uint8, 1),
                ("bEmailOnClr", c_uint8, 1),
                ("bSpareBit5", c_uint8, 1),
                ("bSpareBit6", c_uint8, 1),
                ("bSpareBit7", c_uint8, 1),
                ("bSpareBit8", c_uint8, 1),
               ]


class ChanSetStruct(Structure):

    # typedef struct {
    #     unsigned char AlmDelay;     //Alarm Delay in Mins
    #     unsigned char Scale;        //As Evolution
    #     unsigned char ProbeType;    //As Evolution (MDL_ANA sensortype goes here)
    #     struct {
    #      bool bChan_En:1;       //channel enable flag, true=on.
    #      bool bAlarm_En:1;      //alarm enable flag, true=on.
    #      bool bEmailOnAlm:1;    //email on alarm flag, true=yes.
    #      bool bEmailOnClr:1;    //email on alarm clear, true=yes.
    #      bool bSpareBit5:1;     //Reserved for Future Use
    #      bool bSpareBit6:1;     //Reserved for Future Use
    #      bool bSpareBit7:1;     //Reserved for Future Use
    #      bool bSpareBit8:1;     //Reserved for Future Use
    #      } ChannelF;
    #     } chan_set_t;

    _pack_ = 1
    _fields_ = [
                ("AlmDelay", c_uint8),
                ("Scale", c_uint8),
                ("ProbeType", c_uint8),
                ("ChannelF", ChannelFlagStruct),
               ]


class DstInfoStruct(Structure):

    # //Daylight Saving Time Structure
    # typedef struct {
    #     unsigned long DayLightStart;    //Start of Daylight Savings Time in Seconds
    #     unsigned long StandardStart;    //Start of Standard Savings Time in Seconds
    #     unsigned long DayLightOffset;   //Seconds Time Offset to add in Daylight Savings Time
    #     } dst_info_t;

    _pack_ = 1
    _fields_ = [
                ("DayLightStart", c_uint32),
                ("StandardStart", c_uint32),
                ("DayLightOffset", c_uint32),
               ]


class SysFlagStruct(Structure):

    # typedef struct {
    #     bool bMesher:1;         //Indicates "Always a Mesher" when true.
    #     bool bDisplayOff:1;     //Indicates "Blank Display" when true.
    #     bool bSpareBit3:1;      //Reserved for Future Use
    #     bool bSpareBit4:1;      //Reserved for Future Use
    #     bool bSpareBit5:1;      //Reserved for Future Use
    #     bool bSpareBit6:1;      //Reserved for Future Use
    #     bool bSpareBit7:1;      //Reserved for Future Use
    #     bool bSpareBit8:1;      //Reserved for Future Use
    #     } sysflag_t;

    _pack_ = 1
    _fields_ = [
                ("bMesher", c_uint8, 1),
                ("bDisplayOff", c_uint8, 1),
                ("bSpareBit3", c_uint8, 1),
                ("bSpareBit4", c_uint8, 1),
                ("bSpareBit5", c_uint8, 1),
                ("bSpareBit6", c_uint8, 1),
                ("bSpareBit7", c_uint8, 1),
                ("bSpareBit8", c_uint8, 1),
               ]


class SysSetStruct(Structure):

    # //System Setup Structure (sys_set_t)
    # typedef struct {
    #     dst_info_t DstInfo;
    #     sysflag_t  SetupF;
    #     } sys_set_t;

    _pack_ = 1
    _fields_ = [
                ("DstInfo", DstInfoStruct),
                ("SetupF", SysFlagStruct),
               ]


class TxTaskStruct (Structure):

    # //struct used by Nigel for programming a transmitter. It contains only the data Nigel wants.
    # typedef  struct {
    #     bool          useSimpleAlarms;
    #     limit_t       normal_limits[4]; //Simple alarm zones. Use when "useSimpleAlarms" is true
    #
    #     rate_set_t    rate_set; //*** TIDY-UP LATER ***
    #
    #     chan_set_t    chan_set[4];
    #     sys_set_t     sys_set;
    #     unsigned char DoorDelay;
    # }   txtask_t;

    _pack_ = 1
    _fields_ = [
                ("useSimpleAlarms", c_bool),
                ("normal_limits", LimitStruct * 4),
                ("rate_set", RateSetStruct),
                ("chan_set", ChanSetStruct * 4),
                ("sys_set", SysSetStruct),
                ("DoorDelay", c_uint8),
               ]


class EuConvStruct(Structure):

    # Engineering Units scale conversion stuff for MDL_ANA (RF515)
    #
    # typedef struct {
    #              short MeasLo;      //This is either uA or mV
    #              short MeasHi;      //This is either uA or mV
    #              short EngUnitLo;   //This is the users engineeering units +/- 32000 counts
    #              short EngUnitHi;   //This is the users engineeering units +/- 32000 counts
    #     unsigned  char Decimals;    //This is no. of decimal places in the users engineeering units
    #               char MyScale[11];
    #     } eu_conv_t;

    _pack_ = 1
    _fields_ = [
                ("MeasLo", c_int16),
                ("MeasHi", c_int16),
                ("EngUnitLo", c_int16),
                ("EngUnitHi", c_int16),
                ("Decimals", c_uint8),
                ("MyScale", c_char * 11),
               ]


class AnaTaskStruct (Structure):
    _pack_ = 1
    _fields_ = [
                ("EngScaling", EuConvStruct * 4),
               ]


class CuTaskStruct (Structure):
    _pack_ = 1
    _fields_ = [
                ("description", c_char * 32),
                ("chName", (c_char * 24) * 6),  # Nick's 24 char channel name. NB Maxlen is 24 chars so no zero term
                ("creater", c_char * 32),
                ("location", c_char * 32),
                ("taskNo", c_uint32),
                ("periodNo", c_uint32),
                ("create_date", c_uint32),
                ("anyData", c_bool),
                ("manualStart", c_bool),
                ("useLethCalc", c_bool),
                ("leth_DH", c_float),
                ("leth_T", c_float),
                ("leth_z", c_float),
                ("sno", c_char * 10),
                ("model", c_int16),
                ("issueNo", c_char * 32),
                ("AutoTask", c_bool),
                ("AutoTaskDateValid", c_int8),
                ("AutoTaskDate", c_uint32),
                ("FWTaskDateValid", c_int8),
                ("FWTaskDate", c_uint32),
                ("ana_task", AnaTaskStruct),
                ("skip1", c_char * 549),
                ("anyTask", c_bool),
                ("logging", c_bool),
                ("T_scale", c_uint8),
                ("RH_scale", c_uint8),
                ("skip2", c_uint8 * 4 * 6),
               ]


class ZoneWriteStruct(Structure):

    # typedef struct {
    #     weekday_t     Day;      //The day for which this table is for
    #     chan_t        Chan;     //The Channel for which this table is for
    #     unsigned char Zone[48]; //The actual Table Numbers Values
    #     } zonewr_t;

    _pack_ = 1
    _fields_ = [
                ("Day", c_uint8),
                ("Chan", c_uint8),
                ("Zone", c_uint8 * 48),
               ]


class ZoneTaskStruct(Structure):

    #     typedef  struct {
    #         zonewr_t        zone_set[4][7];
    #     }     ZoneTask_t;

    _pack_ = 1
    _fields_ = [
                ("zone_set", ZoneWriteStruct * 4 * 7)
               ]


class ZoneLimitWriteStruct(Structure):

    # typedef struct {
    #     chan_t        Chan;     //The Channel for which this set of zome limits is for
    #     limit_t       Limit[4]; //The actual Limit Values for [Zone]
    #     } zonelimitwr_t;

    _pack_ = 1
    _fields_ = [
                ("Chan", c_uint8),
                ("Limit", LimitStruct * 4),
               ]


class AlarmZoneLimitsStruct(Structure):

    #     typedef  struct {
    #         zonelimitwr_t   zonelimitwr[4];
    #     }   alarm_zone_limits_t;

    _pack_ = 1
    _fields_ = [
                ("zonelimitwr", ZoneLimitWriteStruct * 4),
               ]


class TaskInfoStruct (Structure):

    # //tot task info
    # typedef  struct {
    #     Zones_Limits_t          Zones_Limits;
    #     txtask_t              TxTask;
    #     CuTask_t              CuTask;
    #     ZoneTask_t             ZoneTask;
    #     alarm_zone_limits_t  alarm_zone_limits;
    # }     taskInfo_t;

    _pack_ = 1
    _fields_ = [
                ("Zones_Limits", ZoneLimitsStruct),
                ("TxTask", TxTaskStruct),
                ("CuTask", CuTaskStruct),
                ("ZoneTask", ZoneTaskStruct),
                ("alarm_zone_limits", AlarmZoneLimitsStruct),
               ]


def load_task(sno, task_no):
    """Load CuTask from Disk - Return a C Structure"""

    taskinfo = TaskInfoStruct()
    filepath = TX_TASK_LIST_FILE_PATTERN % (sno)
    try:
        gwf = open(filepath, 'rb')
    except IOError:
        raise MarvinApiError("File not found: " + filepath)
    while(True):
        number_bytes_read = gwf.readinto(taskinfo)
        if (number_bytes_read == 0 or
            number_bytes_read != sizeof(TaskInfoStruct)):
                raise MarvinApiError("Task %i Not Found: " % task_no + filepath)
        if taskinfo.CuTask.taskNo == task_no:
            break
    gwf.close()
    return taskinfo


def test():
    """Test Me"""
    taskinfo = load_task('CA03110485', 2)
    print taskinfo.CuTask.description
    print taskinfo.CuTask.sno
    print taskinfo.CuTask.chName[0].value
    print taskinfo.CuTask.chName[1].value
#     print gw_id.name
#     print gw_id.version
#     print rd_versions()


if __name__ == '__main__':
    test()
