%import datetime
%from ciphertypes import RdgStatus, RdgCode

%DATE_STRING_FORMAT = "%d %b %Y %H:%M" # Example: 04 Dec 2013 10:41

<%
def style_rdg(rdgcode):
	style = ""
	if rdgcode in (RdgCode.RDG_SKIP, RdgCode.RDG_OVR, RdgCode.RDG_UNR, RdgCode.RDG_FAULT):
		style = "fault"
	end
	if rdgcode == RdgCode.RDG_HIALM or rdgcode == RdgCode.RDG_HITRIG:
		style = "hiAlarm"
	end
	if rdgcode == RdgCode.RDG_LOALM or rdgcode == RdgCode.RDG_LOTRIG:
		style = "loAlarm"
	end
	if rdgcode == RdgCode.RDG_HI:
		style = "hiLimit"
	end
	if rdgcode == RdgCode.RDG_LO:
		style = "loLimit"
	end
	return style
end
%>

<!doctype html>
<head>
<meta content="text/html; charset=UTF-8" http-equiv="content-type">
<title>Tabular Data Report</title>
<link rel="stylesheet" type="text/css" href="/rf500a/Marvin.css" media="all">
<style type="text/css">

h1 {
	text-align: center;
	font-size: 18px;
}

.boxed {
	border-width: 1px;
	border-style: solid;
}

</style>
</head>
<html>
<body>
<div style="width: 100%; text-align: left">
	<div class = "" style="float: left; width: 20%;">
		<img src="/rf500a/images/logo.png" height="53pt" width="128pt" alt="Comark Logo" border="0">
	</div>
	<div class="" style="float: right; width: 20%; text-align: right"> <span>{{datetime.datetime.now().strftime(DATE_STRING_FORMAT)}}</span>
	</div>
	<div class = "" style="clear:none; margin:auto; text-align:center"><span style="font-size:18px; font-weight:bold; line-height:20px">RF500A Tabular Data Report</span> </div>
	<br>
</div>
<div style="clear:both">
<hr>
	<table style="border-collapse: collapse; width: 100%;">
		<tbody>
			<tr>
				<td><b>Transmitter Task Name:</b></td>
				<td>{{task_descr}}</td>
				<td></td>
				<td></td>
				<td><b>Gateway Name:</b></td>
				<td>{{gateway_name}}</td>
			</tr>
			<tr>
				<td><b>Task No:</b></td>
				<td>{{config.task_no}} of {{latest_task}}</td>
				<td></td>
				<td></td>
				<td><b>Gateway Serial No:</b></td>
				<td>{{gateway_sno}}</td>
			</tr>
			<tr>
				%if retrieved_task:
					<td><b>Retrieved:</b></td>
				%else:
					<td><b>Created:</b></td>
				%end
				<td>{{task_create_date.strftime(DATE_STRING_FORMAT)}}</td>
				<td></td>
				<td></td>
				<td><b>Gateway Software:</b></td>
				<td>{{gateway_vn}}</td>
			</tr>
			<tr>
				<td><br></td>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
			</tr>
			<tr>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
				<td><b>Transmitter Serial No:</b></td>
				<td>{{config.sno}}</td>
			</tr>
			<tr>
				<td><b>Starting Alarm Group:</b></td>
				<td>{{start_group}}</td>
				<td></td>
				<td></td>
				<td><b>Model:</b></td>
				<td>{{model}}</td>
			</tr>
			<tr>
				<td><b>Current Alarm Group:</b></td>
				<td>{{current_group}}</td>
				<td></td>
				<td></td>
				<td><b>Transmitter Firmware:</b></td>
				<td>{{version}}</td>
			</tr>
			<tr>
				<td><br></td>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
			</tr>
			<tr>
				<td><b>Report Date Request:</b></td>
				<td>{{rqdate.strftime("%d %b %Y")}}</td>
				<td></td>
				<td></td>
				<td><b>Period No:</b></td>
				<td>{{config.period_no}} of {{latest_period}}</td>
			</tr>
			<tr>
				<td><b>Report Date From:</b></td>
				%if len(txrdgs) == 0:
					<td>None</td>
				%else:
					<td>{{txrdgs[0].rdg_time.strftime(DATE_STRING_FORMAT)}}</td>
				%end
				<td></td>
				<td></td>
				<td><b>Start Time:</b></td>
				<td>{{period_start_date.strftime(DATE_STRING_FORMAT)}}</td>
			</tr>
			<tr>
				<td><b>Report Date To:</b></td>
				%if len(txrdgs) == 0:
					<td>None</td>
				%else:
					<td>{{txrdgs[-1].rdg_time.strftime(DATE_STRING_FORMAT)}}</td>
				%end
				<td></td>
				<td></td>
				<td><b>End Time:</b></td>
				%if period_end_date:
				<td>{{period_end_date.strftime(DATE_STRING_FORMAT)}}</td>
				%else:
				<td>Current Period</td>
				%end
			</tr>
		</tbody>
	</table>
	<br>

	%if datasigned:
	<table border="0" width="95%" height="20" style="border-collapse: collapse" bgcolor="#FFFFFF">
		<tr>
			<td height="20" width="898" align="center"> <p align="center"><font face="Arial" size="2">Data signed by {{signer}}, on {{signdate.strftime(DATE_STRING_FORMAT)}}, reason: {{signreason}} // authority: {{signauth}}</font> </td>
		</tr>
	</table>
	%end

<!-- TABULAR DATA TABLE HEADER -->
%if len(txrdgs) != 0:
	<table class="tabularData" style="table-layout: fixed; width: 100%;" cellspacing="0">
	  <thead>
	    <tr>
	    <th colspan="1" >Date/Time</th>
	%if config.bIntlEn:
		<th colspan="1">INT 1<br>{{chname_intl}}<br></th>
	%end
	%if print_lethality:
		%if config.bExt1En:
		  <th colspan="1" class="channelWithLeathHeader" >EXT 1<br>{{chname_ext1}}<br></th>
		  <th colspan="1" class="leathalityHeader">F Value (mins)<br></th>
		%end
		%if config.bExt2En:
		  <th colspan="1" class="channelWithLeathHeader">EXT 2<br>{{chname_ext2}}<br></th>
		  <th colspan="1" class="leathalityHeader">F Value (mins)<br></th>
		%end
	%else:
		%if config.bExt1En:
		  <th colspan="1">EXT 1<br>{{chname_ext1}}<br></th>
		%end
		%if config.bExt2En:
		  <th colspan="1">EXT 2<br>{{chname_ext2}}<br></th>
		%end
	%end
	%if config.bDoorEn:
		<th colspan="1" class="doorMinsHeader">Door<br>{{chname_door}} (mins)<br></th>
		<th colspan="1" class="doorPercentHeader">%<br></th>
	%end
	    </tr>
	    </thead>
	  <colgroup><col><col><col><col><col><col><col><col></colgroup> <!-- Ooh I didnt adjust this; I am naughty -->
	</table>
%else:
	<h3>No Data</h3>
%end

<!-- TABULAR DATA TABLE BODY -->

<table class="tabularData" style="table-layout: fixed; width: 100%;" cellspacing="0">
<colgroup><col><col><col><col><col><col><col><col></colgroup> <!-- Ooh I didnt adjust this either; I am doubly naughty -->
<tbody>
%for txrdg in txrdgs:
	<tr class="">
		<td class=""><div tabindex="0" style="outline-style:none;">{{txrdg.rdg_time.strftime(DATE_STRING_FORMAT)}}</div></td>
	%if config.bIntlEn:
		<td class="{{style_rdg(txrdg.rdg_status[0].rdgcode)}}"><div style="outline-style:none;">\\
		%if is_rdgcode_graphable(txrdg.rdg_status[0].rdgcode):
			%if txrdg.rdg_status[0].bPeakRdg:
*&nbsp;\\
			%end
{{rdg_formatters[0](txrdg.rdgs[0], config)}}&nbsp;{{intl_scale}}</div></td>
		%else:
{{RdgCode.to_alt_str(txrdg.rdg_status[0].rdgcode)}}</div></td>
		%end
	%end
	%if config.bExt1En:
		<td class="{{style_rdg(txrdg.rdg_status[1].rdgcode)}} channelWithLeathCell"><div style="outline-style:none;">\\
		%if is_rdgcode_graphable(txrdg.rdg_status[1].rdgcode):
			%suppress_leth1 = False
			%if txrdg.rdg_status[1].bPeakRdg:
*&nbsp;\\
			%end
{{rdg_formatters[1](txrdg.rdgs[1], config)}}&nbsp;{{ext1_scale}}</div></td>
		%else:
			%suppress_leth1 = True
{{RdgCode.to_alt_str(txrdg.rdg_status[1].rdgcode)}}</div></td>
		%end
		%if print_lethality:
			<td class="leathalityCell">\\
			%if not suppress_leth1:
				<div style="outline-style:none;">{{"%.0f" % lethality_calculator.calc_next(txrdg, 1)}}</div>
			%end
			</td>
		%end
	%end
	%if config.bExt2En:
		<td class="{{style_rdg(txrdg.rdg_status[2].rdgcode)}} channelWithLeathCell"><div style="outline-style:none;">\\
		%if is_rdgcode_graphable(txrdg.rdg_status[2].rdgcode):
			%suppress_leth2 = False
			%if txrdg.rdg_status[2].bPeakRdg:
*&nbsp;\\
			%end
{{rdg_formatters[2](txrdg.rdgs[2], config)}}&nbsp;{{ext2_scale}}</div></td>
		%else:
			%suppress_leth2 = True
{{RdgCode.to_alt_str(txrdg.rdg_status[2].rdgcode)}}</div></td>
		%end
		%if print_lethality:
			<td class="leathalityCell">\\
			%if not suppress_leth2:
				<div style="outline-style:none;">{{"%.0f" % lethality_calculator.calc_next(txrdg, 2)}}</div>
			%end
			</td>
		%end
	%end
	%if config.bDoorEn:
		<td class="doorMinsCell {{style_rdg(txrdg.rdg_status[3].rdgcode)}}"><div style="outline-style:none;">\\
		%if is_rdgcode_graphable(txrdg.rdg_status[3].rdgcode):
			%if txrdg.rdg_status[3].bPeakRdg:
*&nbsp;\\
			%end
{{rdg_formatters[4](txrdg.rdgs[4], config)}}</div></td>
		%else:
{{RdgCode.to_alt_str(txrdg.rdg_status[3].rdgcode)}}</div></td>
		%end
		<td class="doorPercentCell {{style_rdg(txrdg.rdg_status[3].rdgcode)}}"><div style="outline-style:none;">\\
		%if is_rdgcode_graphable(txrdg.rdg_status[3].rdgcode):
			%if txrdg.rdg_status[3].bPeakRdg:
*&nbsp;\\
			%end
{{rdg_formatters[3](txrdg.rdgs[3], config)}}</div></td>
		%else:
{{RdgCode.to_alt_str(txrdg.rdg_status[3].rdgcode)}}</div></td>
		%end
	%end
	</tr>
%end 
<tbody>
</table>
</br>
</body>
</html>
